@@ -287,9 +287,9 @@ default_file (void)
 /* Keep these sorted!! */
 
 #ifdef USE_PERL
-struct prefs vars[] = {
+const struct prefs vars[] = {
 #else
-static struct prefs vars[] = {
+static const struct prefs vars[] = {
 #endif
 	{"auto_indent", PREFS_OFFINT (auto_indent), TYPE_BOOL},
 	{"auto_resume", PREFS_OFFINT (autoresume), TYPE_BOOL},
@@ -335,11 +335,9 @@ static struct prefs vars[] = {
 	{"dialog_indent_pixels", PREFS_OFFINT (dialog_indent_pixels), TYPE_INT},
 	{"dialog_show_separator", PREFS_OFFINT (dialog_show_separator), TYPE_BOOL},
 	{"dialog_tint", PREFS_OFFINT (dialog_tint), TYPE_BOOL},
-#ifndef WIN32
 	{"dialog_tint_blue", PREFS_OFFINT (dialog_tint_blue), TYPE_INT},
 	{"dialog_tint_green", PREFS_OFFINT (dialog_tint_green), TYPE_INT},
 	{"dialog_tint_red", PREFS_OFFINT (dialog_tint_red), TYPE_INT},
-#endif
 	{"dialog_transparent", PREFS_OFFINT (dialog_transparent), TYPE_BOOL},
 	{"dialog_width", PREFS_OFFINT (dialog_width), TYPE_INT},
 	{"dialog_wordwrap", PREFS_OFFINT (dialog_wordwrap), TYPE_BOOL},
@@ -422,6 +420,9 @@ static struct prefs vars[] = {
 	{"proxy_host", PREFS_OFFSET (proxy_host), TYPE_STR},
 	{"proxy_port", PREFS_OFFINT (proxy_port), TYPE_INT},
 	{"proxy_type", PREFS_OFFINT (proxy_type), TYPE_INT},
+	{"proxy_auth", PREFS_OFFINT (proxy_auth), TYPE_BOOL},
+	{"proxy_user", PREFS_OFFSET (proxy_user), TYPE_STR},
+	{"proxy_pass", PREFS_OFFSET (proxy_pass), TYPE_STR},
 
 	{"quitreason", PREFS_OFFSET (quitreason), TYPE_STR},
 
@@ -455,11 +456,9 @@ static struct prefs vars[] = {
 	{"timestamp_logs", PREFS_OFFINT (timestamp_logs), TYPE_BOOL},
 	{"timestamp_log_format", PREFS_OFFSET (timestamp_log_format), TYPE_STR},
 	{"tint", PREFS_OFFINT (tint), TYPE_BOOL},
-#ifndef WIN32
 	{"tint_blue", PREFS_OFFINT (tint_blue), TYPE_INT},
 	{"tint_green", PREFS_OFFINT (tint_green), TYPE_INT},
 	{"tint_red", PREFS_OFFINT (tint_red), TYPE_INT},
-#endif
 	{"topicbar", PREFS_OFFINT (topicbar), TYPE_BOOL},
 	{"transparent", PREFS_OFFINT (transparent), TYPE_BOOL},
 #ifdef USE_TRANS
@@ -706,7 +705,7 @@ save_config (void)
 }
 
 static void
-set_showval (session *sess, struct prefs *var, char *tbuf)
+set_showval (session *sess, const struct prefs *var, char *tbuf)
 {
 	int len, dots, j;
 	static char *offon[] = { "OFF", "ON" };
