@@ -139,7 +139,7 @@ handle_ctcp (struct session *sess, char *outbuf, char *to, char *nick,
 						 NULL, NULL, 0);
 	} else
 	{
-		chansess = find_session_from_channel (to, sess->server);
+		chansess = find_channel (to, sess->server);
 		if (chansess)
 		{
 			EMIT_SIGNAL (XP_TE_CTCPGENC, chansess, msg, nick, to, NULL, 0);
