@@ -321,14 +321,17 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 	int skip;
 	char *host_n_nick_n_message;
 	GSList *list = dcc_list;
+	session *sess;
 
 	host_n_nick_n_message = malloc (strlen (dcc->nick) + strlen (line) + 26);
 
 	sprintf (host_n_nick_n_message, "%s %d %s: %s",
 				net_ip (dcc->addr), dcc->port, dcc->nick, line);
 
-	skip = perl_dcc_chat (find_session_from_channel (dcc->nick, dcc->serv),
-								 dcc->serv, host_n_nick_n_message);
+	sess = find_dialog (dcc->serv, dcc->nick);
+	if (!sess)
+		sess = dcc->serv->front_session;
+	skip = perl_dcc_chat (sess, host_n_nick_n_message);
 	free (host_n_nick_n_message);
 
 	/* Perl code may have closed the chat. */
@@ -351,7 +354,7 @@ dcc_chat_line (struct DCC *dcc, char *line, char *tbuf)
 		char *po = strchr (line + 8, '\001');
 		if (po)
 			po[0] = 0;
-		sess = find_session_from_channel (dcc->nick, dcc->serv);
+		sess = find_dialog (dcc->serv, dcc->nick);
 		if (!sess)
 			sess = dcc->serv->front_session;
 		channel_action (sess, tbuf, dcc->serv->nick, dcc->nick,
@@ -516,8 +519,11 @@ dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dc
 	char host[128];
 	struct sockaddr_in addr;
 
-	fe_input_remove (dcc->iotag);
-	dcc->iotag = 0;
+	if (dcc->iotag)
+	{
+		fe_input_remove (dcc->iotag);
+		dcc->iotag = 0;
+	}
 
 	memset (&addr, 0, sizeof (addr));
 	addr.sin_port = htons (dcc->port);
@@ -567,20 +573,51 @@ dcc_connect_finished (GIOChannel *source, GIOCondition condition, struct DCC *dc
 	return TRUE;
 }
 
+static int dcc_listen_init (struct DCC *, struct session *);
+
 static void
 dcc_connect (struct DCC *dcc)
 {
+	int ret;
+	char tbuf[400];
+
 	if (dcc->dccstat == STAT_CONNECTING)
 		return;
 	dcc->dccstat = STAT_CONNECTING;
-	dcc->sok = dcc_connect_sok (dcc);
-	if (dcc->sok == -1)
+
+	if (dcc->pasvid)
 	{
-		dcc->dccstat = STAT_FAILED;
-		update_dcc_window (dcc->type);
-		return;
+		/* accepted a passive dcc send */
+		ret = dcc_listen_init (dcc, dcc->serv->front_session);
+		if (!ret)
+		{
+			dcc_close (dcc, STAT_FAILED, FALSE);
+			return;
+		}
+		if (dcc->type == TYPE_RECV)
+			snprintf (tbuf, sizeof (tbuf), strchr (dcc->file, ' ') ?
+					"PRIVMSG %s :\001DCC SEND \"%s\" %lu %d %u %d\001\r\n" :
+					"PRIVMSG %s :\001DCC SEND %s %lu %d %u %d\001\r\n",
+					dcc->nick, dcc->file, dcc->addr, dcc->port, dcc->size,
+					dcc->pasvid);
+		else
+			snprintf (tbuf, sizeof (tbuf),
+						 "PRIVMSG %s :\001DCC CHAT chat %lu %d %d\001\r\n",
+						 dcc->nick, dcc->addr, dcc->port, dcc->pasvid);
+		tcp_send (dcc->serv, tbuf);
 	}
-	dcc->iotag = fe_input_add (dcc->sok, 0, 1, 1, dcc_connect_finished, dcc);
+	else
+	{
+		dcc->sok = dcc_connect_sok (dcc);
+		if (dcc->sok == -1)
+		{
+			dcc->dccstat = STAT_FAILED;
+			update_dcc_window (dcc->type);
+			return;
+		}
+		dcc->iotag = fe_input_add (dcc->sok, 0, 1, 1, dcc_connect_finished, dcc);
+	}
+
 	if (dcc->type == TYPE_RECV)
 		fe_dcc_update_recv (dcc);
 	else
@@ -713,6 +750,10 @@ dcc_accept (GIOChannel *source, GIOCondition condition, struct DCC *dcc)
 	set_nonblocking (sok);
 	dcc->sok = sok;
 	dcc->addr = ntohl (CAddr.sin_addr.s_addr);
+
+	if (dcc->pasvid)
+		return dcc_connect_finished (NULL, 0, dcc);
+
 	dcc->dccstat = STAT_ACTIVE;
 	dcc->lasttime = dcc->starttime = time (0);
 	dcc->fastsend = prefs.fastdccsend;
@@ -912,14 +953,14 @@ dcc_send (struct session *sess, char *tbuf, char *to, char *file)
 						if (havespaces)
 						{
 							snprintf (tbuf, 400,
-									 "PRIVMSG %s :\001DCC SEND \"%s\" %lu %d %d\001\r\n",
+									 "PRIVMSG %s :\001DCC SEND \"%s\" %lu %d %u\001\r\n",
 									 to, file_part (dcc->file), dcc->addr, dcc->port,
 									 dcc->size);
 						}
 						else
 						{
 							snprintf (tbuf, 400,
-									 "PRIVMSG %s :\001DCC SEND %s %lu %d %d\001\r\n",
+									 "PRIVMSG %s :\001DCC SEND %s %lu %d %u\001\r\n",
 									 to, file_part (dcc->file), dcc->addr, dcc->port,
 									 dcc->size);
 						}
@@ -1136,22 +1177,18 @@ dcc_malformed (struct session *sess, char *nick, char *data)
 void
 dcc_resume (struct DCC *dcc)
 {
-	char tbuf[400];
+	char tbuf[410];
 
 	if (dcc->dccstat == STAT_QUEUED && dcc->resumable)
 	{
-		if (strchr (dcc->file, ' ') != NULL)
-		{	/*filename contains spaces*/
-			snprintf (tbuf, sizeof (tbuf),
-						 "PRIVMSG %s :\001DCC RESUME \"%s\" %d %d\001\r\n", dcc->nick,
-						 dcc->file, dcc->port, dcc->resumable);
-		}
-		else
-		{
-			snprintf (tbuf, sizeof (tbuf),
-						 "PRIVMSG %s :\001DCC RESUME %s %d %d\001\r\n", dcc->nick,
-						 dcc->file, dcc->port, dcc->resumable);
-		}
+		snprintf (tbuf, sizeof (tbuf) - 10, strchr (dcc->file, ' ') ?
+					 "PRIVMSG %s :\001DCC RESUME \"%s\" %d %u\001\r\n" :
+					 "PRIVMSG %s :\001DCC RESUME %s %d %u\001\r\n",
+					 dcc->nick, dcc->file, dcc->port, dcc->resumable);
+
+		if (dcc->pasvid)
+			sprintf (tbuf + strlen (tbuf) - 3, " %d\001\r\n", dcc->pasvid);
+
 		tcp_send (dcc->serv, tbuf);
 	}
 }
@@ -1162,15 +1199,19 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 {
 	struct DCC *dcc;
 	char *type = word[5];
-	int port, size;
-	unsigned long addr;
+	int port, pasvid = 0;
+	unsigned long size, addr;
 
 	if (!strcasecmp (type, "CHAT"))
 	{
 		port = atoi (word[8]);
 		sscanf (word[7], "%lu", &addr);
 
-		if (!addr || port < 1024)
+		if (port == 0)
+			pasvid = atoi (word[9]);
+
+		if (!addr || (port < 1024 && port != 0)
+			|| port > 0xffff || (port == 0 && pasvid == 0))
 		{
 			dcc_malformed (sess, nick, word_eol[4] + 2);
 			return;
@@ -1191,6 +1232,7 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 			dcc->dccstat = STAT_QUEUED;
 			dcc->addr = addr;
 			dcc->port = port;
+			dcc->pasvid = pasvid;
 			dcc->nick = strdup (nick);
 			dcc->starttime = time (0);
 
@@ -1213,24 +1255,17 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 			dcc = find_dcc (nick, word[6], TYPE_SEND);
 		if (dcc)
 		{
-			dcc->resumable = atoi (word[8]);
+			sscanf (word[8], "%lu", &size);
+			dcc->resumable = size;
 			if (dcc->resumable < dcc->size)
 			{
 				dcc->pos = dcc->resumable;
 				dcc->ack = dcc->resumable;
 				lseek (dcc->fp, dcc->pos, SEEK_SET);
-				if (strchr (file_part (dcc->file), ' ') != NULL)
-				{	/*filename contains spaces*/
-					snprintf (outbuf, 400,
-								 "PRIVMSG %s :\001DCC ACCEPT \"%s\" %d %d\001\r\n", dcc->nick,
-								 file_part (dcc->file), port, dcc->resumable);
-				}
-				else
-				{
-					snprintf (outbuf, 400,
-								 "PRIVMSG %s :\001DCC ACCEPT %s %d %d\001\r\n", dcc->nick,
-								 file_part (dcc->file), port, dcc->resumable);
-				}
+				snprintf (outbuf, 400, strchr (file_part (dcc->file), ' ') ?
+							 "PRIVMSG %s :\001DCC ACCEPT \"%s\" %d %u\001\r\n" :
+							 "PRIVMSG %s :\001DCC ACCEPT %s %d %u\001\r\n",
+							 dcc->nick, file_part (dcc->file), port, dcc->resumable);
 				tcp_send (dcc->serv, outbuf);
 			}
 			sprintf (outbuf, "%d", dcc->pos);
@@ -1253,11 +1288,15 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 	{
 		char *file = file_part (word[6]);
 		port = atoi (word[8]);
-		size = atoi (word[9]);
 
 		sscanf (word[7], "%lu", &addr);
+		sscanf (word[9], "%lu", &size);
 
-		if (!addr || !size || port < 1024)
+		if (port == 0)
+			pasvid = atoi (word[10]);
+
+		if (!addr || !size || (port < 1024 && port != 0)
+			|| port > 0xffff || (port == 0 && pasvid == 0))
 		{
 			dcc_malformed (sess, nick, word_eol[4] + 2);
 			return;
@@ -1278,7 +1317,18 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 				strcat (dcc->destfile, "/");
 			if (prefs.dccwithnick)
 			{
+#ifdef WIN32
+				char *t = strlen (dcc->destfile) + dcc->destfile;
+				strcpy (t, nick);
+				while (*t)
+				{
+					if (*t == '\\' || *t == '|')
+						*t = '_';
+					t++;
+				}
+#else
 				strcat (dcc->destfile, nick);
+#endif
 				strcat (dcc->destfile, ".");
 			}
 			strcat (dcc->destfile, file);
@@ -1296,6 +1346,7 @@ handle_dcc (struct session *sess, char *outbuf, char *nick, char *word[],
 			dcc->dccstat = STAT_QUEUED;
 			dcc->addr = addr;
 			dcc->port = port;
+			dcc->pasvid = pasvid;
 			dcc->size = size;
 			dcc->nick = strdup (nick);
 			if (prefs.autodccsend)
@@ -1330,7 +1381,7 @@ dontresume:
 			else
 				fe_dcc_update_recv_win ();
 		}
-		sprintf (outbuf, "%d", size);
+		sprintf (outbuf, "%lu", size);
 		EMIT_SIGNAL (XP_TE_DCCSENDOFFER, sess->server->front_session, nick,
 						 file, outbuf, NULL, 0);
 	} else
@@ -1350,7 +1401,7 @@ dcc_show_list (struct session *sess, char *outbuf)
 	{
 		dcc = (struct DCC *) list->data;
 		i++;
-		snprintf (outbuf, 255, " %-5.5s %-10.10s %-7.7s %-7d %-7d %s\n",
+		snprintf (outbuf, 255, " %-5.5s %-10.10s %-7.7s %-7u %-7u %s\n",
 					 dcctypes[(int) dcc->type], dcc->nick,
 					 _(dccstat[(int) dcc->dccstat].name), dcc->size, dcc->pos,
 					 file_part (dcc->file));
