@@ -24,13 +24,12 @@ struct DCC
 	int iotag;
 	int wiotag;
 	int port;
+	int pasvid;
 	int cps;
-	int size;
-	int resumable;
-	int ack;
-	int oldack;
-	int pos;
-	int oldpos;
+	unsigned int size;
+	unsigned int resumable;
+	unsigned int ack;
+	unsigned int pos;
 	time_t starttime;
 	time_t offertime;
 	time_t lasttime;
