@@ -103,7 +103,7 @@ find_session_from_nick (char *nick, server *serv)
 	session *sess;
 	GSList *list = sess_list;
 
-	sess = find_session_from_channel (nick, serv);
+	sess = find_dialog (serv, nick);
 	if (sess)
 		return sess;
 
@@ -133,15 +133,14 @@ find_session_from_nick (char *nick, server *serv)
 }
 
 void
-private_msg (struct server *serv, char *tbuf, char *from, char *ip,
-				 char *text)
+private_msg (server *serv, char *tbuf, char *from, char *ip, char *text)
 {
 	struct session *sess;
 
 	if (EMIT_SIGNAL (XP_PRIVMSG, serv, from, ip, text, NULL, 0) == 1)
 		return;
 
-	sess = find_session_from_channel (from, serv);
+	sess = find_dialog (serv, from);
 
 	if (((sess) && fe_is_beep (sess)) || prefs.beepmsg)
 		fe_beep ();
@@ -244,19 +243,21 @@ channel_action (session *sess, char *tbuf, char *chan, char *from,
 	if (EMIT_SIGNAL (XP_CHANACTION, sess, chan, from, text, NULL, fromme) == 1)
 		return;
 
-	if (is_channel (serv, chan) || fromme)
+	if (!fromme)
 	{
-		sess = find_session_from_channel (chan, serv);
-		if (!fromme && sess)
-			if (fe_is_beep (sess) || prefs.beepchans)
+		if (is_channel (serv, chan))
+		{
+			sess = find_channel (chan, serv);
+			if (sess)
+				if (fe_is_beep (sess) || prefs.beepchans)
+					fe_beep ();
+		} else
+		{
+			/* it's a private action! */
+			sess = find_dialog (serv, from);
+			if (((sess) && fe_is_beep (sess)) || prefs.beepmsg)
 				fe_beep ();
-	}
-	else
-	{
-        	/* it's a private action! */
-		sess = find_session_from_channel (from, serv);
-		if (((sess) && fe_is_beep (sess)) || prefs.beepmsg)
-			fe_beep ();
+		}
 	}
 
 	if (!sess && !is_channel (serv, chan) && prefs.autodialog)
@@ -300,7 +301,9 @@ channel_msg (struct server *serv, char *outbuf, char *chan, char *from,
 	int hilight = FALSE;
 	char nickchar[2] = "\000";
 
-	sess = find_session_from_channel (chan, serv);
+	sess = find_channel (chan, serv);
+	if (!sess && !is_channel (serv, chan))
+		sess = find_dialog (serv, chan);
 	if (!sess)
 		return;
 
@@ -449,7 +452,7 @@ you_joined (server *serv, char *outbuf, char *chan, char *nick, char *ip)
 	session *sess;
 
 	/* already joined? probably a bnc */
-	sess = find_session_from_channel (chan, serv);
+	sess = find_channel (chan, serv);
 	if (!sess)
 	{
 		/* see if a window is waiting to join this channel */
@@ -497,7 +500,7 @@ static void
 you_kicked (struct server *serv, char *tbuf, char *chan, char *kicker,
 				char *reason)
 {
-	struct session *sess = find_session_from_channel (chan, serv);
+	struct session *sess = find_channel (chan, serv);
 	if (sess)
 	{
 		EMIT_SIGNAL (XP_TE_UKICK, sess, serv->nick, chan, kicker, reason, 0);
@@ -517,7 +520,7 @@ you_kicked (struct server *serv, char *tbuf, char *chan, char *kicker,
 static void
 you_parted (struct server *serv, char *chan, char *ip, char *reason)
 {
-	struct session *sess = find_session_from_channel (chan, serv);
+	struct session *sess = find_channel (chan, serv);
 	if (sess)
 	{
 		if (*reason)
@@ -536,7 +539,7 @@ names_list (server *serv, char *chan, char *names)
 	char name[NICKLEN];
 	int pos = 0;
 
-	sess = find_session_from_channel (chan, serv);
+	sess = find_channel (chan, serv);
 	if (!sess)
 	{
 		EMIT_SIGNAL (XP_TE_USERSONCHAN, serv->front_session, chan, names, NULL,
@@ -585,7 +588,7 @@ topic (server *serv, char *buf)
 	if (po)
 	{
 		po[0] = 0;
-		sess = find_session_from_channel (buf, serv);
+		sess = find_channel (buf, serv);
 		if (sess)
 		{
 			new_topic = strip_color (po + 2);
@@ -599,7 +602,7 @@ topic (server *serv, char *buf)
 static void
 new_topic (server *serv, char *nick, char *chan, char *topic)
 {
-	struct session *sess = find_session_from_channel (chan, serv);
+	struct session *sess = find_channel (chan, serv);
 	if (sess)
 	{
 		set_topic (sess, topic);
@@ -610,7 +613,7 @@ new_topic (server *serv, char *nick, char *chan, char *topic)
 static void
 user_joined (server *serv, char *chan, char *user, char *ip)
 {
-	struct session *sess = find_session_from_channel (chan, serv);
+	struct session *sess = find_channel (chan, serv);
 
 	if (EMIT_SIGNAL (XP_JOIN, serv, chan, user, ip, NULL, 0) == 1)
 		return;
@@ -626,7 +629,7 @@ user_joined (server *serv, char *chan, char *user, char *ip)
 static void
 user_kicked (server *serv, char *chan, char *user, char *kicker, char *reason)
 {
-	struct session *sess = find_session_from_channel (chan, serv);
+	struct session *sess = find_channel (chan, serv);
 	if (sess)
 	{
 		EMIT_SIGNAL (XP_TE_KICK, sess, kicker, user, chan, reason, 0);
@@ -638,7 +641,7 @@ static void
 user_parted (struct server *serv, char *chan, char *user, char *ip,
 				 char *reason)
 {
-	struct session *sess = find_session_from_channel (chan, serv);
+	struct session *sess = find_channel (chan, serv);
 	if (sess)
 	{
 		if (!fe_is_confmode (sess))
@@ -666,7 +669,7 @@ topic_nametime (server *serv, char *chan, char *nick, char *date)
 {
 	long n = atol (date);
 	char *tim = ctime (&n);
-	struct session *sess = find_session_from_channel (chan, serv);
+	struct session *sess = find_channel (chan, serv);
 	tim[19] = 0;	/* get rid of the \n */
 	EMIT_SIGNAL (XP_TE_TOPICDATE, sess, chan, nick, tim, NULL, 0);
 }
@@ -787,12 +790,12 @@ notice (struct server *serv, char *outbuf, char *to, char *nick, char *msg,
 	struct session *sess = 0;
 
 	if (is_channel (serv, ptr))
-		sess = find_session_from_channel (ptr, serv);
+		sess = find_channel (ptr, serv);
 
 	if (!sess && ptr[0] == '@')
 	{
 		ptr++;
-		sess = find_session_from_channel (ptr, serv);
+		sess = find_channel (ptr, serv);
 	}
 	
 	if (!sess)
@@ -890,7 +893,7 @@ end_of_names (server *serv, char *chan)
 		}
 		return TRUE;
 	}
-	sess = find_session_from_channel (chan, serv);
+	sess = find_channel (chan, serv);
 	if (sess)
 	{
 		sess->end_of_names = TRUE;
@@ -1124,7 +1127,7 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 		fe_chan_list_end (sess->server);
 		break;
 	case 324:
-		sess = find_session_from_channel (word[4], serv);
+		sess = find_channel (word[4], serv);
 		if (!sess)
 			sess = serv->front_session;
 		if (sess->ignore_mode)
@@ -1146,7 +1149,7 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 		handle_mode (serv, outbuf, word, word_eol, "", TRUE);
 		break;
 	case 329:
-		sess = find_session_from_channel (word[4], serv);
+		sess = find_channel (word[4], serv);
 		if (sess)
 		{
 			if (sess->ignore_date)
@@ -1174,7 +1177,7 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 		{
 			struct session *who_sess;
 
-			who_sess = find_session_from_channel (word[4], serv);
+			who_sess = find_channel (word[4], serv);
 			if (who_sess)
 			{
 				sprintf (outbuf, "%s@%s", word[5], word[6]);
@@ -1216,7 +1219,7 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 		} else
 		{
 			struct session *who_sess;
-			who_sess = find_session_from_channel (word[4], serv);
+			who_sess = find_channel (word[4], serv);
 			if (who_sess)
 			{
 				if (who_sess->doing_who)
@@ -1254,7 +1257,7 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 			time_str[19] = 0;	/* get rid of the \n */
 			if (ban_time == 0)
 				time_str = "";
-			sess = find_session_from_channel (word[4], serv);
+			sess = find_channel (word[4], serv);
 			if (!sess)
 				sess = serv->front_session;
 		   if (!fe_is_banwindow (sess))
@@ -1264,7 +1267,7 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 		}
 		break;
 	 case 368:
-		sess = find_session_from_channel (word[4], serv);
+		sess = find_channel (word[4], serv);
 		if (!sess)
 			sess = serv->front_session;
 		if (!fe_is_banwindow (sess))
@@ -1356,7 +1359,7 @@ process_numeric (server * serv, session * sess, char *outbuf, int n,
 				chan += strlen (serv->nick) + 1;
 			if (*chan == ':')
 				chan++;
-			realsess = find_session_from_channel (chan, serv);
+			realsess = find_channel (chan, serv);
 			if (!realsess)
 				realsess = sess;
 			EMIT_SIGNAL (XP_TE_SERVTEXT, realsess, text, NULL, NULL, NULL, 0);
@@ -1408,7 +1411,7 @@ process_line (struct server *serv, char *buf)
 #ifdef USE_PERL
 	/* tell perl that we got a line of data */
 	if (*buf == ':' && is_channel (serv, word[3]))
-		tmp_sess = find_session_from_channel (word[3], serv);
+		tmp_sess = find_channel (word[3], serv);
 	else
 		tmp_sess = sess;
 
