@@ -182,7 +182,7 @@ handle_single_mode (server * serv, char *outbuf, char sign, char mode,
 	outbuf[2] = mode;
 	outbuf[3] = 0;
 
-	sess = find_session_from_channel (chan, serv);
+	sess = find_channel (chan, serv);
 	if (!sess || !is_channel (serv, chan))
 	{
 		/* got modes for a chan we're not in! probably nickmode +isw etc */
@@ -209,7 +209,7 @@ handle_single_mode (server * serv, char *outbuf, char sign, char mode,
 		case 'k':
 			if (EMIT_SIGNAL (XP_CHANSETKEY, sess, chan, nick, arg, NULL, 0) == 1)
 				return;
-			strncpy (sess->channelkey, arg, 60);
+			safe_strcpy (sess->channelkey, arg, sizeof (sess->channelkey));
 			fe_update_channel_key (sess);
 			fe_update_mode_buttons (sess, mode, sign);
 			if (!quiet)
@@ -408,7 +408,7 @@ handle_mode (server * serv, char *outbuf, char *word[], char *word_eol[],
 	if (*modes == 0)
 		return;	/* beyondirc's blank modes */
 
-	sess = find_session_from_channel (chan, serv);
+	sess = find_channel (chan, serv);
 	if (!sess)
 		sess = serv->front_session;
 	/* remove trailing space */
@@ -489,7 +489,7 @@ handle_005 (server * serv, char *word[])
 	char *pre;
 
 	w = 4;							  /* start at the 4th word */
-	while (*word[w] && w < PDIWORDS)
+	while (w < PDIWORDS && *word[w])
 	{
 		if (strncmp (word[w], "MODES=", 6) == 0)
 		{
@@ -524,6 +524,11 @@ handle_005 (server * serv, char *word[])
 		} else if (strncmp (word[w], "WATCH=", 6) == 0)
 		{
 			serv->supports_watch = TRUE;
+		} else if (strncmp (word[w], "NETWORK=", 8) == 0)
+		{
+			if (serv->network)
+				free (serv->network);
+			serv->network = strdup (word[w] + 8);
 		}
 
 		w++;
