@@ -280,7 +280,7 @@ notify_send_watches (server * serv)
 		if (len > 500)
 		{
 			notify_flush_watches (serv, point, list);
-			len = 0;
+			len = strlen (notify->name) + 2;
 			point = list;
 		}
 
