@@ -908,17 +908,23 @@ ban (session * sess, char *tbuf, char *mask, char *bantypestr, int deop)
 		    mask[0] == '=' ||
 		    mask[0] == '^' ||
 		    mask[0] == '-')
-			strcpy (username, mask+1);
-		else
-			strcpy (username, mask);
+		{
+			safe_strcpy (username, mask+1, sizeof (username));
+		} else
+		{
+			safe_strcpy (username, mask, sizeof (username));
+		}
 		*at = '@';
-		strcpy (fullhost, at + 1);
+		safe_strcpy (fullhost, at + 1, sizeof (fullhost));
 
 		dot = strchr (fullhost, '.');
 		if (dot)
-			strcpy (domain, dot);
-		else
-			strcpy (domain, fullhost);
+		{
+			safe_strcpy (domain, dot, sizeof (domain));
+		} else
+		{
+			safe_strcpy (domain, fullhost, sizeof (domain));
+		}
 
 		if (*bantypestr)
 			bantype = atoi (bantypestr);
@@ -1492,7 +1498,7 @@ cmd_dns (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			}
 		} else
 		{
-			sprintf (tbuf, "/exec %s %s", prefs.dnsprogram, nick);
+			sprintf (tbuf, "/exec -d %s %s", prefs.dnsprogram, nick);
 			handle_command (tbuf, sess, 0, 0);
 		}
 		return TRUE;
@@ -1800,30 +1806,41 @@ cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 	char *cmd = word_eol[2];
 	int fds[2], pid = 0;
 	struct nbexec *s;
+	int shell = TRUE;
 
 	if (*cmd)
 	{
-		if (access ("/bin/sh", X_OK) != 0)
-		{
-			fe_message (_("I need /bin/sh to run!\n"), FALSE);
-			return TRUE;
-		}
 		exec_check_process (sess);
 		if (sess->running_exec != NULL)
 		{
 			EMIT_SIGNAL (XP_TE_ALREADYPROCESS, sess, NULL, NULL, NULL, NULL, 0);
 			return TRUE;
 		}
-		if (!strcmp (word[2], "-o"))
+
+		if (!strcmp (word[2], "-d"))
 		{
 			if (!*word[3])
 				return FALSE;
-			else
+			cmd = word_eol[3];
+			shell = FALSE;
+		}
+		else if (!strcmp (word[2], "-o"))
+		{
+			if (!*word[3])
+				return FALSE;
+			cmd = word_eol[3];
+			tochannel = TRUE;
+		}
+
+		if (shell)
+		{
+			if (access ("/bin/sh", X_OK) != 0)
 			{
-				cmd = word_eol[3];
-				tochannel = TRUE;
+				fe_message (_("I need /bin/sh to run!\n"), FALSE);
+				return TRUE;
 			}
 		}
+
 #ifdef __EMX__						  /* if os/2 */
 		if (pipe (fds) < 0)
 		{
@@ -1860,7 +1877,17 @@ cmd_exec (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 			/* Also copy it to stdin so we can write to it */
 			dup2 (fds[1], 0);
 			/* Now we call /bin/sh to run our cmd ; made it more friendly -DC1 */
-			execl ("/bin/sh", "sh", "-c", cmd, 0);
+			if (shell)
+			{
+				execl ("/bin/sh", "sh", "-c", cmd, 0);
+			} else
+			{
+				char **argv;
+				int argc;
+
+				my_poptParseArgvString (cmd, &argc, &argv);
+				execvp (argv[0], argv);
+			}
 			/* not reached unless error */
 			/*printf("exec error\n");*/
 			fflush (stdout);
@@ -2337,7 +2364,9 @@ cmd_msg (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 				if (sess->lastnick[0])
 					nick = sess->lastnick;
 			} else
-				strcpy (sess->lastnick, nick);	/* prime the last nick memory */
+			{
+				safe_strcpy (sess->lastnick, nick, sizeof (sess->lastnick));	/* prime the last nick memory */
+			}
 
 			if (*nick == '=')
 			{
@@ -2357,7 +2386,9 @@ cmd_msg (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 				sprintf (tbuf, "PRIVMSG %s :%s\r\n", nick, msg);
 				tcp_send (sess->server, tbuf);
 			}
-			newsess = find_session_from_channel (nick, sess->server);
+			newsess = find_dialog (sess->server, nick);
+			if (!newsess)
+				newsess = find_channel (nick, sess->server);
 			if (newsess)
 				channel_msg (newsess->server, tbuf, newsess->channel,
 						newsess->server->nick, msg, TRUE);
@@ -2485,6 +2516,8 @@ cmd_part (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 		chan = sess->channel;
 	if ((*chan) && is_channel (sess->server, chan))
 	{
+		if (reason[0] == 0)
+			reason = NULL;
 		server_sendpart (sess->server, chan, reason);
 		return TRUE;
 	}
@@ -2513,7 +2546,7 @@ int
 cmd_query (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
 	char *nick = word[2];
-	if (*nick)
+	if (*nick && !is_channel (sess->server, nick))
 	{
 		if (!find_dialog (sess->server, nick))
 			new_ircwindow (sess->server, nick, SESS_DIALOG);
@@ -2677,6 +2710,7 @@ cmd_server (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 
 	if (strncasecmp ("irc://", server, 6) == 0)
 	{
+		;
 		server += 6;
 #ifdef USE_OPENSSL
 urlserv:
@@ -2702,14 +2736,16 @@ urlserv:
 			} else
 			{
 				sess->willjoinchannel[0] = '#';
-				strncpy (sess->willjoinchannel + 1, co, CHANLEN);
+				strncpy (sess->willjoinchannel + 1, co, CHANLEN - 1);
 				sess->willjoinchannel[CHANLEN-1] = 0;
 			}
 		}
 	}
 
 	if (*pass)
-		strcpy (sess->server->password, pass);
+	{
+		safe_strcpy (sess->server->password, pass, sizeof (sess->server->password));
+	}
 #ifdef USE_OPENSSL
 	sess->server->use_ssl = use_ssl;
 	sess->server->accept_invalid_cert = TRUE;
@@ -2742,7 +2778,7 @@ cmd_servchan (struct session *sess, char *tbuf, char *word[],
 
 	if (*word[4 + offset])
 	{
-		strcpy (sess->willjoinchannel, word[4 + offset]);
+		safe_strcpy (sess->willjoinchannel, word[4 + offset], CHANLEN);
 		return cmd_server (sess, tbuf, word, word_eol);
 	}
 
@@ -2940,10 +2976,8 @@ cmd_voice (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 int
 cmd_flushq (struct session *sess, char *tbuf, char *word[], char *word_eol[])
 {
-	char buf[128];
-
-	sprintf (buf, "Flushing server send queue, %d bytes.\n", sess->server->sendq_len);
-	PrintText (sess, buf);
+	sprintf (tbuf, "Flushing server send queue, %d bytes.\n", sess->server->sendq_len);
+	PrintText (sess, tbuf);
 	flush_server_queue (sess->server);
 	return TRUE;
 }
@@ -3354,7 +3388,9 @@ handle_command (char *cmd, session *sess, int history, int nocommand)
 	if (prefs.nickcompletion)
 		perform_nick_completion (sess, cmd, newcmd);
 	else
-		strncpy (newcmd, cmd, sizeof (newcmd));
+	{
+		safe_strcpy (newcmd, cmd, sizeof (newcmd));
+	}
 
 	cmd = newcmd;
 
