@@ -45,7 +45,9 @@
 #include "outbound.h"
 #include "xchatc.h"
 #include "perlc.h"
-
+#ifdef ENABLE_NLS
+#include <locale.h>
+#endif
 
 struct _perl_timeout_handlers
 {
@@ -343,6 +345,7 @@ perl_init (void)
 
 	my_perl = perl_alloc ();
 	perl_construct (my_perl);
+	PL_perl_destruct_level = 1;
 	if (prefs.perlwarnings)
 		perl_parse (my_perl, xs_init, 4, perl_args, NULL);
 	else
@@ -491,7 +494,7 @@ perl_inbound (struct session *sess, struct server *serv, char *buf, char *msg_ty
 }
 
 int
-perl_dcc_chat (struct session *sess, struct server *serv, char *buf)
+perl_dcc_chat (struct session *sess, char *buf)
 {
 	GSList *handler;
 	struct _perl_inbound_handlers *data;
@@ -1244,7 +1247,10 @@ static XS (XS_IRC_dcc_list)
 		i++;
 		XST_mIV (i, dcc->size);
 		i++;
-		XST_mIV (i, dcc->resumable);
+		if (dcc->type == TYPE_SEND)
+			XST_mIV (i, dcc->pos);
+		else
+			XST_mIV (i, dcc->resumable);
 		i++;
 		XST_mIV (i, dcc->addr);
 		i++;
