@@ -653,8 +653,8 @@ server_stopconnecting (server * serv)
 	kill (serv->childpid, SIGKILL);
 	waitpid (serv->childpid, NULL, 0);
 #else
-	TerminateThread ((HANDLE)serv->childpid, 0);
-	CloseHandle ((HANDLE)serv->childpid);
+	PostThreadMessage (serv->childpid, WM_QUIT, 0, 0);
+	CloseHandle ((HANDLE)serv->childhandle);
 #endif
 
 	close (serv->childwrite);
@@ -897,14 +897,69 @@ traverse_wingate (int sok, char *serverAddr, int port)
 	return 0;
 }
 
+static void
+three_to_four (char *from, char *to)
+{
+	static const char tab64[64]={  'A','B','C','D','E','F','G','H',
+					 'I','J','K','L','M','N','O','P',
+					 'Q','R','S','T','U','V','W','X',
+					 'Y','Z','a','b','c','d','e','f',
+					 'g','h','i','j','k','l','m','n',
+					 'o','p','q','r','s','t','u','v',
+					 'w','x','y','z','0','1','2','3',
+					 '4','5','6','7','8','9','+','/'
+	};
+
+	to[0] = tab64 [ (from[0] >> 2) & 63 ];
+	to[1] = tab64 [ ((from[0] << 4) | (from[1] >> 4)) & 63 ];
+	to[2] = tab64 [ ((from[1] << 2) | (from[2] >> 6)) & 63 ];
+	to[3] = tab64 [ from[2] & 63 ];
+};
+
+static void
+base64_encode (char *to, char *from, unsigned int len)
+{
+	while (len >= 3)
+	{
+		three_to_four (from, to);
+		len -= 3;
+		from += 3;
+		to += 4;
+	}
+	if (len)
+	{
+		char three[3]={0,0,0};
+		int i=0;
+		for (i=0;i<len;i++)
+			three[i] = *from++;
+		three_to_four (three, to);
+		if (len == 1)
+			to[2] = to[3] = '=';
+		else if (len == 2)
+			to[3] = '=';
+		to += 4;
+	};
+	to[0] = 0;
+}
+
 static int
 traverse_http (int sok, char *serverAddr, int port)
 {
-	char buf[128];
-	int n;
+	char buf[256];
+	char auth_data[128];
+	char auth_data2[68];
+	int n, n2;
 
-	n = snprintf (buf, sizeof (buf), "CONNECT %s:%d HTTP/1.0\r\n\r\n",
+	n = snprintf (buf, sizeof (buf), "CONNECT %s:%d HTTP/1.0\r\n",
 					  serverAddr, port);
+	if (prefs.proxy_auth)
+	{
+		n2 = snprintf (auth_data2, sizeof (auth_data2), "%s:%s",
+							prefs.proxy_user, prefs.proxy_pass);
+		base64_encode (auth_data, auth_data2, n2);
+		n += snprintf (buf+n, sizeof (buf)-n, "Proxy-Authorization: Basic %s\r\n", auth_data);
+	}
+	n += snprintf (buf+n, sizeof (buf)-n, "\r\n");
 	send (sok, buf, n, 0);
 #ifndef WIN32
 	waitline (sok, buf, sizeof (buf)); /* FIXME: win32 cant read() sok */
@@ -1064,6 +1119,7 @@ xit:
 		free (ip);
 	if (real_hostname)
 		free (real_hostname);
+	fclose (fd);
 #endif
 
 	return 0;
@@ -1075,6 +1131,17 @@ connect_server (session * sess, char *hostname, int port, int no_login)
 	int pid, read_des[2];
 	server *serv = sess->server;
 
+#ifdef USE_OPENSSL
+	if (!ctx && serv->use_ssl)
+	{
+		if (!(ctx = _SSL_context_init (ssl_cb_info, FALSE)))
+		{
+			fprintf (stderr, "_SSL_context_init failed\n");
+			exit (1);
+		}
+	}
+#endif
+
 	if (!hostname[0])
 		return;
 
@@ -1112,8 +1179,9 @@ connect_server (session * sess, char *hostname, int port, int no_login)
 	net_sockets (&serv->sok4, &serv->sok6);
 
 #ifdef WIN32
-	pid = (int)CreateThread (NULL, 0, (LPTHREAD_START_ROUTINE)server_child,
-									 serv, 0, (DWORD *)&pid);
+	serv->childhandle = (int)CreateThread (NULL, 0,
+													(LPTHREAD_START_ROUTINE)server_child,
+													serv, 0, (DWORD *)&pid);
 #else
 	switch (pid = fork ())
 	{
