@@ -154,13 +154,12 @@ PrintText (struct session *sess, unsigned char *text)
 	{
 		time_t timval = time (0);
 		char *temp = strip_color (text);
-		char *tim = malloc (32);
+		char tim[32];
 		strftime (tim, 32, prefs.timestamp_log_format, localtime (&timval));
 		if (prefs.timestamp || prefs.timestamp_logs)
 			write (sess->logfd, tim, strlen(tim));
 		write (sess->logfd, temp, strlen (temp));
 		free (temp);
-		free (tim);
 	}
 
 #ifdef USE_HEBREW
@@ -199,7 +198,7 @@ end_logging (int fd)
 }
 
 static int
-open_log_file (char *servname, char *channame)
+open_log_file (char *servname, char *channame, char *netname)
 {
 	char buf[512], fn[256];
 	char *dir, *sep;
@@ -234,7 +233,7 @@ open_log_file (char *servname, char *channame)
 #else
 		mkdir (buf, S_IRUSR | S_IWUSR | S_IXUSR);
 #endif
-	auto_insert (fn, prefs.logmask, NULL, NULL, "", channame, "", "", "", servname);
+	auto_insert (fn, prefs.logmask, NULL, NULL, "", channame, "", "", netname, servname);
 
 	snprintf (buf, 510, "%s/xchatlogs/%s", get_xdir (), fn);
 
@@ -286,7 +285,7 @@ open_log_file (char *servname, char *channame)
 #else
 	fd = open (buf, O_CREAT | O_APPEND | O_WRONLY | OFLAGS, 0644);
 #endif
-	if (fd < 0)
+	if (fd == -1)
 		return -1;
 	currenttime = time (NULL);
 	write (fd, buf,
@@ -301,7 +300,7 @@ setup_logging (struct session *sess)
 {
 	if (sess->logfd != -1)
 		end_logging (sess->logfd);
-	sess->logfd = open_log_file (sess->server->servername, sess->channel);
+	sess->logfd = open_log_file (sess->server->servername, sess->channel, sess->server->network);
 }
 
 /* Print Events stuff here --AGL */
@@ -1371,8 +1370,8 @@ load_text_events ()
 static void
 display_event (char *i, struct session *sess, int numargs, char **args)
 {
-	int len, oi, ii, *ar;
-	char o[4096], d, a, done_all = FALSE;
+	int len, oi, ii;
+	char *ar, o[4096], d, a, done_all = FALSE;
 
 	oi = ii = len = d = a = 0;
 
@@ -1405,13 +1404,13 @@ display_event (char *i, struct session *sess, int numargs, char **args)
 							a, numargs, i);
 				break;
 			}
-			ar = (int *) args[(int) a];
+			ar = args[(int) a];
 			if (ar == NULL)
 			{
 				printf ("Error args[a] == NULL in display_event\n");
 				abort ();
 			}
-			len = strlen ((char *) ar);
+			len = strlen (ar);
 			memcpy (&o[oi], ar, len);
 			oi += len;
 			break;
