@@ -246,11 +246,11 @@ file_part (char *file)
 }
 
 void
-path_part (char *file, char *path)
+path_part (char *file, char *path, int pathlen)
 {
 	char *filepart = file_part (file);
 	*filepart = 0;
-	strcpy (path, file);
+	safe_strcpy (path, file, pathlen);
 }
 
 char *				/* like strstr(), but nocase */
@@ -966,7 +966,7 @@ country (char *hostname)
 /* this is taken from gnome-libs 1.2.4 */
 #define POPT_ARGV_ARRAY_GROW_DELTA 5
 
-static int my_poptParseArgvString(const char * s, int * argcPtr, char *** argvPtr) {
+int my_poptParseArgvString(const char * s, int * argcPtr, char *** argvPtr) {
     char * buf, * bufStart, * dst;
     const char * src;
     char quote = '\0';
@@ -1078,8 +1078,8 @@ util_exec (char *cmd)
 #else
 	spawnvp (_P_DETACH, argv[0], argv);
 	free (argv);
-	return 0;
 #endif
+	return 0;
 }
 
 unsigned long
@@ -1144,7 +1144,7 @@ rfc_ncasecmp (char *str1, char *str2, int n)
 	return (res);
 }
 
-unsigned char tolowertab[] =
+const unsigned char tolowertab[] =
 	{ 0, 0x1, 0x2, 0x3, 0x4, 0x5, 0x6, 0x7, 0x8, 0x9, 0xa,
 	0xb, 0xc, 0xd, 0xe, 0xf, 0x10, 0x11, 0x12, 0x13, 0x14,
 	0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d,
