@@ -105,12 +105,13 @@ tcp_send_real (struct server *serv, char *buf, int len)
 			tbuf = sbuf;
 		if (!tbuf)
 			return -1;
-		strcpy (tbuf, buf);
+		memcpy (tbuf, buf, len); /* buf is not necessarily NULL terminated! */
+		tbuf[len] = 0;
 		user2serv (tbuf);
 	}
 #endif
 
-	fe_add_rawlog (serv, tbuf, TRUE);
+	fe_add_rawlog (serv, tbuf, len, TRUE);
 
 	if (!EMIT_SIGNAL (XP_IF_SEND, GINT_TO_POINTER (serv->sok), tbuf,
 							GINT_TO_POINTER (len), NULL, NULL, 0))
@@ -192,48 +193,45 @@ tcp_send_queue (struct server *serv)
 	return 0;						  /* remove the timeout handler */
 }
 
-#ifdef USE_JCODE
-static void
-fe_add_ja_rawlog (struct server *serv, char *buf,  int outbound)
-{
-	char *jbuf =  kanji_conv_to_locale(buf);
-	if (jbuf)
-	{
-		fe_add_rawlog (serv, jbuf, TRUE);
-		free(jbuf);
-	} else
-			fe_add_rawlog (serv, buf, TRUE);
-}
-#endif
-
 int
-tcp_send_len (struct server *serv, char *buf, int len)
+tcp_send_len (server *serv, char *buf, int len)
 {
-    char *dbuf;
-    int noqueue = !serv->outbound_queue;
-#ifdef USE_JCODE
-    unsigned char *jbuf = NULL;
+	char *dbuf;
+	int noqueue = !serv->outbound_queue;
 
-    if ( prefs.kanji_conv ) {
-        jbuf = kanji_conv_auto(buf, "ISO-2022-JP");
-        if (!jbuf)
-            return 0;
-        else {
-            buf = strdup(jbuf); free(jbuf);
-            len = strlen(buf);
-        }
-    } else
-        buf = strdup(buf); /* freed after all */
-#endif    
+#ifdef USE_JCODE
+	if (prefs.kanji_conv)
+	{
+		buf = kanji_conv_auto (buf, "ISO-2022-JP");
+		if (!buf)
+			return 0;
+		len = strlen (buf);
+	}
+
+	if (!prefs.throttle)
+	{
+		int ret = tcp_send_real (serv, buf, len);
+		if (prefs.kanji_conv)
+			free (buf);
+		return ret;
+	}
+#else
 
 	if (!prefs.throttle)
 		return tcp_send_real (serv, buf, len);
 
+#endif
+
 	dbuf = malloc (len + 2);	/* first byte is the priority */
 	dbuf[0] = 1;	/* pri 1 for most things */
 	memcpy (dbuf + 1, buf, len);
 	dbuf[len + 1] = 0;
 
+#ifdef USE_JCODE
+	if (prefs.kanji_conv)
+		free (buf);
+#endif
+
 	/* privmsg and notice get a lower priority */
 	if (strncasecmp (dbuf + 1, "PRIVMSG", 7) == 0 ||
 		 strncasecmp (dbuf + 1, "NOTICE", 6) == 0)
@@ -244,7 +242,7 @@ tcp_send_len (struct server *serv, char *buf, int len)
 
 	if (tcp_send_queue (serv) && noqueue)
 		fe_timeout_add (500, tcp_send_queue, serv);
-    
+
 	return 1;
 }
 
@@ -298,14 +296,15 @@ find_dialog (struct server *serv, char *nick)
 }
 
 session *
-find_session_from_channel (char *chan, server *serv)
+find_channel (char *chan, server *serv)
 {
 	session *sess;
 	GSList *list = sess_list;
 	while (list)
 	{
 		sess = list->data;
-		if (sess->type != SESS_SHELL && !strcasecmp (chan, sess->channel))
+		if (sess->type != SESS_SHELL && sess->type != SESS_DIALOG &&
+			 !strcasecmp (chan, sess->channel))
 		{
 			if (!serv || serv == sess->server)
 				return sess;
@@ -425,7 +424,6 @@ lag_check (void)
 				auto_reconnect (serv, FALSE, -1);
 			} else
 			{
-				/*sprintf (tbuf, "PING LAG%lu :%s\r\n", tim, serv->servername);*/
 				sprintf (tbuf, "PING LAG%lu\r\n", tim);
 				tcp_send (serv, tbuf);
 				serv->lag_sent = tim;
@@ -524,11 +522,14 @@ set_server_defaults (server *serv)
 		free (serv->nick_prefixes);
 	if (serv->nick_modes)
 		free (serv->nick_modes);
+	if (serv->network)
+		free (serv->network);
 
 	serv->chantypes = strdup ("#&!+");
 	serv->chanmodes = strdup ("beI,k,l");
 	serv->nick_prefixes = strdup ("@%+");
 	serv->nick_modes = strdup ("ohv");
+	serv->network = strdup (serv->servername);
 
 	serv->nickcount = 1;
 	serv->end_of_motd = FALSE;
@@ -611,6 +612,7 @@ kill_server_callback (server * serv)
 	free (serv->nick_prefixes);
 	free (serv->chanmodes);
 	free (serv->chantypes);
+	free (serv->network);
 	if (serv->bad_nick_prefixes)
 		free (serv->bad_nick_prefixes);
 	if (serv->last_away_reason)
@@ -985,6 +987,7 @@ save_away_message (struct server *serv, char *nick, char *msg)
 	"NAME SUB\n"								"CMD Mozilla...\n\n"\
    	"NAME Open in existing\n"			"CMD !mozilla -remote 'openURL(%s)'\n\n"\
    	"NAME Open in new window\n"		"CMD !mozilla -remote 'openURL(%s,new-window)'\n\n"\
+		"NAME Open in new tab\n"			"CMD !mozilla -remote 'openURL(%s,new-tab)'\n\n"\
    	"NAME Run new Mozilla\n"			"CMD !mozilla %s\n\n"\
 	"NAME ENDSUB\n"							"CMD \n\n"\
 	"NAME SUB\n"								"CMD Galeon...\n\n"\
@@ -1012,6 +1015,29 @@ save_away_message (struct server *serv, char *nick, char *msg)
 	"NAME Connect as IRC server\n"		"CMD /newserver %s\n\n")
 #endif
 
+
+#ifndef WIN32
+/* Close and open log files on SIGUSR1. Usefull for log rotating */
+
+static void 
+sighup_handler (int signal)
+{
+	GSList *list = sess_list;
+	session *sess;
+
+	if (prefs.logging)
+	{
+		while (list)
+		{
+			sess = list->data;
+			setup_logging (sess);
+			list = list->next;
+		}
+	}
+}
+
+#endif
+
 static void
 xchat_init (void)
 {
@@ -1027,6 +1053,9 @@ xchat_init (void)
 	act.sa_flags = 0;
 	sigemptyset (&act.sa_mask);
 	sigaction (SIGPIPE, &act, NULL);
+
+	/* Deal with SIGUSR1's */
+	signal (SIGUSR1, sighup_handler);
 #endif
 
 	signal_setup ();
@@ -1118,13 +1147,6 @@ main (int argc, char *argv[])
 	SOCKSinit (argv[0]);
 #endif
 
-#ifdef USE_OPENSSL
-	if (!(ctx = _SSL_context_init (ssl_cb_info, FALSE))) {
-		fprintf(stderr, "_SSL_context_init failed\n");
-		exit(1);
-	}
-#endif
-
 	if (!fe_args (argc, argv))
 		return 0;
 
@@ -1141,7 +1163,8 @@ main (int argc, char *argv[])
 #endif
 
 #ifdef USE_OPENSSL
-	_SSL_context_free (ctx);
+	if (ctx)
+		_SSL_context_free (ctx);
 #endif
 
 	return 0;
