@@ -126,6 +126,9 @@ struct xchatprefs
 	char proxy_host[64];
 	int proxy_port;
 	int proxy_type; /* 0=disabled, 1=wingate 2=socks4, 3=socks5, 4=http */
+	unsigned int proxy_auth;
+	char proxy_user[32];
+	char proxy_pass[32];
 
 	int first_dcc_send_port;
 	int last_dcc_send_port;
@@ -337,6 +340,9 @@ struct server
 	int childread;
 	int childwrite;
 	int childpid;
+#ifdef WIN32
+	int childhandle;
+#endif
 	int iotag;
 	int bartag;
 	int recondelay_tag;				/* reconnect delay timeout */
@@ -376,6 +382,7 @@ struct server
 	time_t lag_sent;
 	time_t ping_recv;					/* when we last got a ping reply */
 	time_t away_time;					/* when we were marked away */
+	char *network;
 
 	int motd_skipped:1;
 	int connected:1;
