@@ -21,7 +21,7 @@ extern GSList *urlhandler_list;
 
 int tcp_send_len (server *serv, char *buf, int len);
 int tcp_send (server *serv, char *buf);
-session * find_session_from_channel (char *chan, server *serv);
+session * find_channel (char *chan, server *serv);
 session * find_dialog (server *serv, char *nick);
 session * new_ircwindow (server *serv, char *name, int type);
 void set_server_defaults (server *serv);
