@@ -105,7 +105,7 @@ banlist_unban (GtkWidget * wid, struct session *sess)
 	i = 0;
 	while (list)
 	{
-		row = (int) list->data;
+		row = GPOINTER_TO_INT (list->data);
 		gtk_clist_get_text (GTK_CLIST (sess->gui->banlist_clistBan), row, 0,
 								  &masks[i]);
 		i++;
@@ -139,7 +139,7 @@ banlist_crop (GtkWidget * wid, struct session *sess)
 	{
 		for (i = 0; i < sel_length; i++)
 		{
-			row = (int) g_list_nth_data (oldsel, i);
+			row = GPOINTER_TO_INT (g_list_nth_data (oldsel, i));
 			gtk_clist_unselect_row (GTK_CLIST (sess->gui->banlist_clistBan), row,
 											0);
 		}
@@ -171,7 +171,11 @@ banlist_opengui (struct session *sess)
    GtkWidget *butUnbanEntry;
 */
 	char tbuf[256];
-	gchar *titles[] = { _("Mask"), _("From"), _("Date") };
+	gchar *titles[3];
+
+	titles[0] = _("Mask");
+	titles[1] = _("From");
+	titles[2] = _("Date");
 
 	if (sess->gui->banlist_window)
 	{
