@@ -44,11 +44,14 @@
 #ifndef WIN32
 
 static int
-reg_match (regex_t * regexptr, const char *str)
+reg_match (server *serv, const char *str)
 {
 	int m;
 
-	m = regexec (regexptr, str, 1, NULL, REG_NOTBOL);
+	if (!serv->gui->have_regex)
+		return 0;
+
+	m = regexec (&serv->gui->chanlist_match_regex, str, 1, NULL, REG_NOTBOL);
 
 	/* regex returns 0 if it's a match: */
 	return m == 0 ? 1 : 0;
@@ -57,9 +60,9 @@ reg_match (regex_t * regexptr, const char *str)
 #else
 
 static int
-reg_match (char ** regexptr, const char *str)
+reg_match (server *serv, const char *str)
 {
-	return match (*regexptr, str);
+	return match (serv->gui->chanlist_match_regex, str);
 }
 
 #endif
@@ -188,6 +191,8 @@ static void
 chanlist_place_row_in_gui (struct server *serv, gchar ** next_row)
 {
 	int num_users = atoi (next_row[1]);
+	gfloat val, end;
+	GtkAdjustment *adj;
 
 	/* First, update the 'found' counter values */
 	serv->gui->chanlist_users_found_count += num_users;
@@ -217,8 +222,8 @@ chanlist_place_row_in_gui (struct server *serv, gchar ** next_row)
 			 (!serv->gui->chanlist_match_wants_channel
 			  && !serv->gui->chanlist_match_wants_topic))
 		{
-			if (!reg_match (&serv->gui->chanlist_match_regex, next_row[0])
-				 && !reg_match (&serv->gui->chanlist_match_regex, next_row[2]))
+			if (!reg_match (serv, next_row[0])
+				 && !reg_match (serv, next_row[2]))
 			{
 				chanlist_update_caption (serv);
 				return;
@@ -227,7 +232,7 @@ chanlist_place_row_in_gui (struct server *serv, gchar ** next_row)
 
 		else if (serv->gui->chanlist_match_wants_channel)
 		{
-			if (!reg_match (&serv->gui->chanlist_match_regex, next_row[0]))
+			if (!reg_match (serv, next_row[0]))
 			{
 				chanlist_update_caption (serv);
 				return;
@@ -236,7 +241,7 @@ chanlist_place_row_in_gui (struct server *serv, gchar ** next_row)
 
 		else if (serv->gui->chanlist_match_wants_topic)
 		{
-			if (!reg_match (&serv->gui->chanlist_match_regex, next_row[2]))
+			if (!reg_match (serv, next_row[2]))
 			{
 				chanlist_update_caption (serv);
 				return;
@@ -244,6 +249,8 @@ chanlist_place_row_in_gui (struct server *serv, gchar ** next_row)
 		}
 	}
 
+	adj = gtk_clist_get_vadjustment (GTK_CLIST (serv->gui->chanlist_list));
+	val = adj->value;
 	/*
 	 * If all the above above tests passed or if no text was in the 
 	 * chanlist_wild_text, add this entry to the GUI
@@ -254,6 +261,12 @@ chanlist_place_row_in_gui (struct server *serv, gchar ** next_row)
 	serv->gui->chanlist_users_shown_count += num_users;
 	serv->gui->chanlist_channels_shown_count++;
 
+	/* restore original scrollbar position */
+	end = adj->upper - adj->lower - adj->page_size;
+	if (val > end)
+		val = end;
+	gtk_adjustment_set_value (adj, val);
+
 	chanlist_update_caption (serv);
 }
 
@@ -373,21 +386,23 @@ chanlist_apply_pressed (GtkButton * button, struct server *serv)
 static void
 chanlist_wild (GtkWidget * wid, struct server *serv)
 {
+	const char *pattern = gtk_entry_get_text (GTK_ENTRY (wid));
+
 	/* Store the pattern text in the wild_text var so next time the window is 
 	 * opened it is remembered. 
 	 */
-	strncpy (serv->gui->chanlist_wild_text,
-				gtk_entry_get_text (GTK_ENTRY (wid)), 255);
+	safe_strcpy (serv->gui->chanlist_wild_text, pattern,
+					 sizeof (serv->gui->chanlist_wild_text));
 
 #ifdef WIN32
-	serv->gui->chanlist_match_regex =
-		strdup (gtk_entry_get_text (GTK_ENTRY (wid)));
+	serv->gui->chanlist_match_regex = strdup (pattern);
 #else
 	/* recompile the regular expression. */
-	regfree (&serv->gui->chanlist_match_regex);
-	regcomp (&serv->gui->chanlist_match_regex,
-				gtk_entry_get_text (GTK_ENTRY (wid)),
+	if (serv->gui->have_regex)
+		regfree (&serv->gui->chanlist_match_regex);
+	regcomp (&serv->gui->chanlist_match_regex, pattern,
 				REG_ICASE | REG_EXTENDED | REG_NOSUB);
+	serv->gui->have_regex = 1;
 #endif
 }
 
@@ -546,7 +561,11 @@ chanlist_destroy_widget (GtkObject * object, struct server *serv)
 {
 	chanlist_data_free (serv);
 #ifndef WIN32
-	regfree (&serv->gui->chanlist_match_regex);
+	if (serv->gui->have_regex)
+	{
+		regfree (&serv->gui->chanlist_match_regex);
+		serv->gui->have_regex = 0;
+	}
 #else
 	free (serv->gui->chanlist_match_regex);
 #endif
@@ -562,11 +581,15 @@ chanlist_closegui (server *serv)
 void
 chanlist_opengui (struct server *serv)
 {
-	gchar *titles[] = { _("Channel"), _("Users"), _("Topic") };
+	gchar *titles[3];
 	GtkWidget *frame, *vbox, *hbox, *sortvbox, *sorthbox, *numtable,
 		*table, *wid, *real_wid;
 	char tbuf[256];
 
+	titles[0] = _("Channel");
+	titles[1] = _("Users");
+	titles[2] = _("Topic");
+
 	if (serv->gui->chanlist_window)
 	{
 		gdk_window_show (serv->gui->chanlist_window->window);
