@@ -43,13 +43,6 @@ struct dccwindow
 	GtkWidget *list;
 };
 
-struct dcc_send
-{
-	struct session *sess;
-	char nick[64];
-	GtkWidget *freq;
-};
-
 static struct dccwindow dccrwin;	/* recv */
 static struct dccwindow dccswin;	/* send */
 static struct dccwindow dcccwin;	/* chat */
@@ -95,7 +88,7 @@ fe_dcc_update_recv (struct DCC *dcc)
 	/* percentage done */
 	per = (float) ((dcc->pos * 100.00) / dcc->size);
 
-	snprintf (pos, sizeof (pos), "%d", dcc->pos);
+	snprintf (pos, sizeof (pos), "%u", dcc->pos);
 	snprintf (cps, sizeof (cps), "%d", dcc->cps);
 	snprintf (perc, sizeof (perc), "%.0f%%", per);
 
@@ -143,8 +136,8 @@ fe_dcc_update_send (struct DCC *dcc)
 	per = (float) ((dcc->ack * 100.00) / dcc->size);
 
 	snprintf (pos, sizeof (pos), "%d", dcc->pos);
-	snprintf (cps, sizeof (cps), "%d", dcc->cps);
-	snprintf (ack, sizeof (ack), "%d", dcc->ack);
+	snprintf (cps, sizeof (cps), "%u", dcc->cps);
+	snprintf (ack, sizeof (ack), "%u", dcc->ack);
 	snprintf (perc, sizeof (perc), "%.0f%%", per);
 	if (dcc->cps != 0)
 	{
@@ -237,11 +230,11 @@ fe_dcc_update_recv_win (void)
 			else
 				nnew[8] = "";
 #endif
-			sprintf (size, "%d", dcc->size);
+			sprintf (size, "%u", dcc->size);
 			if (dcc->dccstat == STAT_QUEUED)
-				sprintf (pos, "%d", dcc->resumable);
+				sprintf (pos, "%u", dcc->resumable);
 			else
-				sprintf (pos, "%d", dcc->pos);
+				sprintf (pos, "%u", dcc->pos);
 			snprintf (cps, sizeof (cps), "%d", dcc->cps);
 			/* percentage recv'ed */
 			per = (float) ((dcc->pos * 100.00) / dcc->size);
@@ -273,7 +266,7 @@ dcc_info (struct DCC *dcc)
 	char tbuf[256];
 	snprintf (tbuf, 255, _("      File: %s\n"
 				 "   To/From: %s\n"
-				 "      Size: %d\n"
+				 "      Size: %u\n"
 				 "      Port: %d\n"
 				 " IP Number: %s\n"
 				 "Start Time: %s"),
@@ -399,12 +392,21 @@ fe_dcc_open_recv_win (int passive)
 {
 	GtkWidget *vbox, *bbox;
 #ifdef USE_GNOME
-	gchar *titles[] =
-		{ _("Status"), _("File"), _("Size"), _("Position"), "%", _("CPS"), _("ETA"), _("From"),
-			_("MIME Type") };
+	gchar *titles[9];
 #else
-	gchar *titles[] =
-		{ _("Status"), _("File"), _("Size"), _("Position"), "%", _("CPS"), _("ETA"), _("From") };
+	gchar *titles[8];
+#endif
+
+	titles[0] = _("Status");
+	titles[1] = _("File");
+	titles[2] = _("Size");
+	titles[3] = _("Position");
+	titles[4] = "%";
+	titles[5] = _("CPS");
+	titles[6] = _("ETA");
+	titles[7] = _("From");
+#ifdef USE_GNOME
+	titles[8] = _("MIME Type");
 #endif
 
 	if (dccrwin.window)
@@ -502,11 +504,11 @@ fe_dcc_update_send_win (void)
 			nnew[8] = dcc->nick;
 			/* percentage ack'ed */
 			per = (float) ((dcc->ack * 100.00) / dcc->size);
-			snprintf (size, sizeof (size), "%d", dcc->size);
-			snprintf (pos, sizeof (pos), "%d", dcc->pos);
+			snprintf (size, sizeof (size), "%u", dcc->size);
+			snprintf (pos, sizeof (pos), "%u", dcc->pos);
 			snprintf (cps, sizeof (cps), "%d", dcc->cps);
 			snprintf (perc, sizeof (perc), "%.0f%%", per);
-			snprintf (ack, sizeof (ack), "%d", dcc->ack);
+			snprintf (ack, sizeof (ack), "%u", dcc->ack);
 			if (dcc->cps != 0)
 			{
 				to_go = (dcc->size - dcc->ack) / dcc->cps;
@@ -583,9 +585,17 @@ void
 fe_dcc_open_send_win (int passive)
 {
 	GtkWidget *vbox, *bbox;
-	gchar *titles[] =
-		{ _("Status"), _("File"), _("Size"), _("Position"), _("Ack"), "%", _("CPS"), _("ETA"),
-			_("To") };
+	gchar *titles[9];
+
+	titles[0] = _("Status");
+	titles[1] = _("File");
+	titles[2] = _("Size");
+	titles[3] = _("Position");
+	titles[4] = _("Ack");
+	titles[5] = "%";
+	titles[6] = _("CPS");
+	titles[7] = _("ETA");
+	titles[8] = _("To");
 
 	if (dccswin.window)
 	{
@@ -691,8 +701,8 @@ fe_dcc_update_chat_win (void)
 		{
 			nnew[0] = _(dccstat[(int) dcc->dccstat].name);
 			nnew[1] = dcc->nick;
-			sprintf (pos, "%d", dcc->pos);
-			sprintf (siz, "%d", dcc->size);
+			sprintf (pos, "%u", dcc->pos);
+			sprintf (siz, "%u", dcc->size);
 			nnew[4] = ctime (&dcc->starttime);
 			row = gtk_clist_append (GTK_CLIST (dcccwin.list), nnew);
 			gtk_clist_set_row_data (GTK_CLIST (dcccwin.list), row,
@@ -714,8 +724,13 @@ void
 fe_dcc_open_chat_win (int passive)
 {
 	GtkWidget *vbox, *bbox;
-	gchar *titles[] =
-		{ _("Status"), _("To/From"), _("Recv"), _("Sent"), _("StartTime") };
+	gchar *titles[5];
+
+	titles[0] = _("Status");
+	titles[1] = _("To/From");
+	titles[2] = _("Recv");
+	titles[3] = _("Sent");
+	titles[4] = _("StartTime");
 
 	if (dcccwin.window)
 	{
