@@ -375,9 +375,15 @@ key_dialog_close ()
 static void
 key_dialog_add_new (GtkWidget * button, GtkCList * list)
 {
-	gchar *strs[] = { "", _("<none>"), _("<none>"), _("<none>"), _("<none>") };
+	gchar *strs[5];
 	struct key_binding *kb;
 
+	strs[0] = "";
+	strs[1] = _("<none>");
+	strs[2] = _("<none>");
+	strs[3] = _("<none>");
+	strs[4] = _("<none>");
+
 	kb = malloc (sizeof (struct key_binding));
 
 	kb->keyval = 0;
@@ -618,10 +624,14 @@ key_dialog_show ()
 {
 	GtkWidget *vbox, *hbox, *list, *vbox2, *wid, *wid2, *wid3, *hbox2;
 	struct key_binding *kb;
-	gchar *titles[] = { _("Mod"), _("Key"), _("Action"), "1", "2" };
+	gchar *titles[] = { NULL, NULL, NULL, "1", "2" };
 	char temp[32];
 	int i;
 
+	titles[0] = _("Mod");
+	titles[1] = _("Key");
+	titles[2] = _("Action");
+
 	if (key_dialog)
 	{
 		wins_bring_tofront (key_dialog);
@@ -1850,7 +1860,9 @@ tab_nick_comp (GtkWidget *t, int shift)
 		nick_buf[0] = 0;
 		while (match_list)
 		{
-			sprintf (nick_buf, "%s%s ", nick_buf, (char *)match_list->data);
+			snprintf (nick_buf, sizeof (nick_buf), "%s%s ", nick_buf,
+						 (char *)match_list->data);
+			nick_buf[sizeof (nick_buf) - 1] = 0;
 			match_list = match_list->next;
 		}
 		PrintText (sess, nick_buf);
