@@ -25,7 +25,6 @@
 #include "fe-gtk.h"
 #include "../common/xchat.h"
 #include "../common/ignore.h"
-#include "../common/plugin.h"
 #include "../common/cfgfiles.h"
 #include "gtkutil.h"
 #include "wins.h"
@@ -327,11 +326,18 @@ ignore_stats_entry (GtkWidget * box, char *label, int value)
 void
 ignore_gui_open ()
 {
-	gchar *titles[] =
-		{ _("Mask"), _("CTCP"), _("Private"), _("Chan"), _("Notice"), _("Invite"), _("Unignore") };
+	gchar *titles[7];
 	GtkWidget *vbox, *box, *wid, *stat_box, *frame;
 	GSList *temp = ignore_list;
 
+	titles[0] = _("Mask");
+	titles[1] = _("CTCP");
+	titles[2] = _("Private");
+	titles[3] = _("Chan");
+	titles[4] = _("Notice");
+	titles[5] = _("Invite");
+	titles[6] = _("Unignore");
+
 	if (ignorewin)
 	{
 		wins_bring_tofront (ignorewin);
