@@ -66,7 +66,7 @@ GtkStyle *redtab_style = 0;
 GtkStyle *bluetab_style;
 GtkStyle *inputgad_style;
 
-static char chan_flags[] = { 't', 'n', 's', 'i', 'p', 'm', 'l', 'k' };
+static const char chan_flags[] = { 't', 'n', 's', 'i', 'p', 'm', 'l', 'k' };
 
 static void userlist_button (GtkWidget * box, char *label, char *cmd,
 									  struct session *sess, int a, int b, int c,
@@ -193,13 +193,16 @@ maingui_createbuttons (session * sess)
 void
 fe_buttons_update (struct session *sess)
 {
-	if (sess->gui->button_box)
+	if (sess->type != SESS_DIALOG)
 	{
-		gtk_widget_destroy (sess->gui->button_box);
-		sess->gui->button_box = 0;
+		if (sess->gui->button_box)
+		{
+			gtk_widget_destroy (sess->gui->button_box);
+			sess->gui->button_box = 0;
+		}
+		if (prefs.userlistbuttons)
+			maingui_createbuttons (sess);
 	}
-	if (prefs.userlistbuttons)
-		maingui_createbuttons (sess);
 }
 
 void
@@ -1254,7 +1257,7 @@ new_tree_view ()
 	return tree;
 }
 
-void
+static void
 tree_update ()
 {
 	GList *tree;
@@ -1750,6 +1753,7 @@ create_window (struct session *sess)
 	GtkWidget *leftpane, *rightpane;
 	GtkWidget *vbox, *tbox, *bbox, *nlbox, *wid, *infbox;
 	int justopened = FALSE;
+	gint screen_width, screen_height;
 
 	if (sess->type == SESS_DIALOG)
 	{
@@ -2046,6 +2050,16 @@ create_window (struct session *sess)
 		if (justopened)
 		{
 			gtk_widget_show (main_window);
+
+			screen_width = gdk_screen_width ();
+			screen_height = gdk_screen_height ();
+
+			if (prefs.mainwindow_left > screen_width)
+				prefs.mainwindow_left = (screen_width - prefs.mainwindow_width);
+
+			if (prefs.mainwindow_top > screen_height)
+				prefs.mainwindow_top = (screen_height - prefs.mainwindow_height);
+
 			if (prefs.mainwindow_left || prefs.mainwindow_top)
 				gdk_window_move (main_window->window,
 									  prefs.mainwindow_left, prefs.mainwindow_top);
