@@ -148,7 +148,12 @@ void
 notify_opengui (void)
 {
 	GtkWidget *vbox, *bbox;
-	gchar *titles[] = { _("User"), _("Status"), _("Server"), _("Last Seen") };
+	gchar *titles[4];
+
+	titles[0] = _("User");
+	titles[1] = _("Status");
+	titles[2] = _("Server");
+	titles[3] = _("Last Seen");
 
 	if (notify_window)
 	{
