@@ -185,6 +185,8 @@ add_defaults (void)
 		strcpy (serv.comment, dserv[i].comment);
 		serv.port = dserv[i].port;
 		serv.flags = 0;
+		if (serv.port == 9999)
+			serv.flags |= USE_SSL;
 		add_server_entry (&serv);
 		i++;
 	}
@@ -434,7 +436,7 @@ delete_server_clicked (GtkWidget * wid, struct session *sess)
  * close_edit_entry:
  * called when we want to destroy the edit entry win
  */
-static void
+static gboolean
 close_edit_entry (void)
 {
 	if (sleditwin)
@@ -450,6 +452,8 @@ close_edit_entry (void)
 	editentry = 0;
 	editnode = 0;
 	new_entry = 0;
+
+	return FALSE;
 }
 
 /*
@@ -1150,15 +1154,15 @@ group_expand (GtkCTree * tree, GtkCTreeNode * node)
 	}
 }
 
-static int
-serverlist_close_cb (GtkWidget *wid, gpointer unused)
+static gboolean
+serverlist_close_cb (GtkWidget *wid, GdkEvent *event, gpointer unused)
 {
 	close_server_list (FALSE);
 
 	if (sess_list == NULL)
 		xchat_exit ();
 
-	return 0;
+	return FALSE;
 }
 
 /*
