@@ -46,14 +46,17 @@ static const struct defaultserv dserv[] =
 	{"", "ENDSUB", "", 0},
 
 	{"", "SUB", "EFNet", 0},
-	{"", "us.rr.efnet.net", "EFNet USA", 6667},
-	{"", "ca.rr.efnet.net", "EFNet Canada", 6667},
-	{"", "eu.rr.efnet.net", "EFNet Europe", 6667},
-	{"", "au.rr.efnet.net", "EFNet Australia", 6667},
-	{"", "irc.efnet.org", "irc.efnet.org", 6667},
-	{"", "irc.light.se", "irc.light.se", 6667},
-	{"", "irc.stanford.edu", "irc.stanford.edu", 6667},
-	{"", "irc.solidstreaming.net", "irc.solidstreaming.net", 6667},
+	{"", "irc.Prison.NET", "irc.Prison.NET", 6667},
+	{"", "irc.Qeast.net", "irc.Qeast.net", 6667},
+	{"", "irc.efnet.pl", "irc.efnet.pl", 6667},
+	{"", "irc.flamed.net", "irc.flamed.net", 6667},
+	{"", "efnet.demon.co.uk", "efnet.demon.co.uk", 6667},
+	{"", "irc.lagged.org", "irc.lagged.org", 6667},
+	{"", "irc.lightning.net", "irc.lightning.net", 6667},
+	{"", "irc.mindspring.com", "irc.mindspring.com", 6667},
+	{"", "irc.rt.ru", "irc.rt.ru", 6667},
+	{"", "irc.easynews.com", "irc.easynews.com", 6667},
+	{"", "irc.servercentral.net", "irc.servercentral.net", 6667},
 	{"", "ENDSUB", "", 0},
 
 	{"", "SUB", "GalaxyNet", 0},
@@ -488,6 +491,25 @@ static const struct defaultserv dserv[] =
 	{"", "blender.evolnet.org", "blender.evolnet.org", 6667},
 	{"", "ENDSUB", "", 0},
 
+	{"", "SUB", "irc.oftc.net", 0},
+	{"", "irc.oftc.net", "irc.oftc.net", 6667},
+	{"", "ENDSUB", "", 0},
+
+	{"", "SUB", "Xentonix.net", 0},
+	{"", "irc.ffm.de.eu.xentonix.net", "irc.ffm.de.eu.xentonix.net", 6667},
+	{"", "irc.brs.de.eu.xentonix.net", "irc.brs.de.eu.xentonix.net", 6667},
+	{"", "irc.stg.ch.eu.xentonix.net", "irc.stg.ch.eu.xentonix.net", 6667},
+	{"", "irc.hou.tx.us.xentonix.net", "irc.hou.tx.us.xentonix.net", 6667},
+	{"", "irc.kar.de.eu.xentonix.net", "irc.kar.de.eu.xentonix.net", 6667},
+	{"", "irc.vie.at.eu.xentonix.net", "irc.vie.at.eu.xentonix.net", 6667},
+	{"", "ENDSUB", "", 0},
+
+	{"", "SUB", "Spidernet.org", 0},
+	{"", "us.spidernet.org", "Random US server", 6667},
+	{"", "eu.spidernet.org", "Random European server", 6667},
+	{"", "irc.spidernet.org", "Random server", 6667},
+	{"", "ENDSUB", "", 0},
+
 /*	{"", "SUB", "", 0},
 	{"", "", "", 6667},
 	{"", "ENDSUB", "", 0},*/
