@@ -69,6 +69,9 @@ struct setup
 
 	GtkWidget *entry_proxy_host;
 	GtkWidget *entry_proxy_port;
+	GtkWidget *check_proxy_auth;
+	GtkWidget *entry_proxy_user;
+	GtkWidget *entry_proxy_pass;
 
 	GtkWidget *entry_dcc_send_port_first;
 	GtkWidget *entry_dcc_send_port_last;
@@ -536,6 +539,8 @@ settings_ok_clicked (GtkWidget * wid, struct session *sess)
 	GETSTR (sess->setup->prefs.dcc_ip_str, entry_dcc_ip_str);
 	GETSTR (sess->setup->prefs.hostname, entry_hostname);
 	GETSTR (sess->setup->prefs.proxy_host, entry_proxy_host);
+	GETSTR (sess->setup->prefs.proxy_user, entry_proxy_user);
+	GETSTR (sess->setup->prefs.proxy_pass, entry_proxy_pass);
 	sess->setup->prefs.proxy_port = GETINT (entry_proxy_port);
 
 	sess->setup->prefs.first_dcc_send_port = GETINT (entry_dcc_send_port_first);
@@ -605,6 +610,7 @@ settings_ok_clicked (GtkWidget * wid, struct session *sess)
 									 notify_checklist, 0);
 	}
 
+#ifndef WIN32
 	if (prefs.use_fontset != sess->setup->prefs.use_fontset)
 	{
 		/* force font reloading */
@@ -612,6 +618,7 @@ settings_ok_clicked (GtkWidget * wid, struct session *sess)
 		prefs.dialog_font_normal[0] = 0;
 		prefs.use_fontset = sess->setup->prefs.use_fontset;
 	}
+#endif
 
 	if (strcmp (prefs.font_normal, sess->setup->prefs.font_normal) != 0)
 	{
@@ -833,6 +840,15 @@ settings_pinvtoggle_check (GtkWidget * widget, int *pref)
 
 /* Misc callbacks */
 
+static void
+settings_proxy_auth_check (GtkWidget * widget, struct session *sess)
+{
+	int active = GTK_TOGGLE_BUTTON (widget)->active;
+	sess->setup->prefs.proxy_auth = active;
+	gtk_widget_set_sensitive (sess->setup->entry_proxy_user, active);
+	gtk_widget_set_sensitive (sess->setup->entry_proxy_pass, active);
+}
+
 static void
 settings_transparent_check (GtkWidget * widget, struct session *sess)
 {
@@ -840,7 +856,7 @@ settings_transparent_check (GtkWidget * widget, struct session *sess)
 	{
 		sess->setup->prefs.transparent = TRUE;
 		gtk_widget_set_sensitive (sess->setup->background, FALSE);
-#if defined(USE_GDK_PIXBUF) || defined(USE_MMX)
+#if defined(USE_GDK_PIXBUF) || defined(USE_MMX) || defined(WIN32)
 		gtk_widget_set_sensitive (sess->setup->check_tint, TRUE);
 #endif
 	} else
@@ -848,7 +864,9 @@ settings_transparent_check (GtkWidget * widget, struct session *sess)
 		sess->setup->prefs.transparent = FALSE;
 		gtk_toggle_button_set_active ((GtkToggleButton *) sess->setup->
 												check_tint, FALSE);
+#ifndef WIN32
 		gtk_widget_set_sensitive (sess->setup->background, TRUE);
+#endif
 		gtk_widget_set_sensitive (sess->setup->check_tint, FALSE);
 	}
 }
@@ -860,7 +878,7 @@ settings_transparent_dialog_check (GtkWidget * widget, struct session *sess)
 	{
 		sess->setup->prefs.dialog_transparent = TRUE;
 		gtk_widget_set_sensitive (sess->setup->background_dialog, FALSE);
-#if defined(USE_GDK_PIXBUF) || defined(USE_MMX)
+#if defined(USE_GDK_PIXBUF) || defined(USE_MMX) || defined(WIN32)
 		gtk_widget_set_sensitive (sess->setup->dialog_check_tint, TRUE);
 #endif
 	} else
@@ -868,7 +886,9 @@ settings_transparent_dialog_check (GtkWidget * widget, struct session *sess)
 		sess->setup->prefs.dialog_transparent = FALSE;
 		gtk_toggle_button_set_active ((GtkToggleButton *) sess->setup->
 												dialog_check_tint, FALSE);
+#ifndef WIN32
 		gtk_widget_set_sensitive (sess->setup->background_dialog, TRUE);
+#endif
 		gtk_widget_set_sensitive (sess->setup->dialog_check_tint, FALSE);
 	}
 }
@@ -975,7 +995,9 @@ settings_create_window_page (GtkWidget * vvbox,
 								  backgroundwid, 0, settings_openfiledialog,
 								  row_index++);
 #endif
+#ifndef WIN32
 	if (transparent)
+#endif
 		gtk_widget_set_sensitive (*backgroundwid, FALSE);
 
 	wid = settings_create_toggle (_("Indent Nicks"), left_box, *indent_nicks,
@@ -1002,22 +1024,24 @@ settings_create_window_page (GtkWidget * vvbox,
 	*tintwid = settings_create_toggle (_("Tint Transparency"), right_box, tint,
 												  settings_ptoggle_check, tint_pref);
 	add_tip (*tintwid, _("Tint the see-through text box to make it darker"));
+#ifndef WIN32
 	settings_create_toggle (_("Use a font set"),
 									left_box, prefs.use_fontset, settings_ptoggle_check,
 									&(sess->setup->prefs.use_fontset));
+#endif
 
-#ifndef USE_XLIB
+#if !defined(USE_XLIB) && !defined(WIN32)
 	gtk_widget_set_sensitive (*transwid, FALSE);
 #endif
 
-#if defined(USE_GDK_PIXBUF) || defined(USE_MMX)
+#if defined(USE_GDK_PIXBUF) || defined(USE_MMX) || defined(WIN32)
 	if (!transparent)
 #endif
 		gtk_widget_set_sensitive (*tintwid, FALSE);
 
 	vbox = settings_create_group (vvbox, _("Tint Setting"));
 
-#if !defined(USE_GDK_PIXBUF) && !defined(USE_MMX)
+#if !defined(USE_GDK_PIXBUF) && !defined(USE_MMX) && !defined(WIN32)
 	gtk_widget_set_sensitive (vbox, FALSE);
 #endif
 
@@ -1507,6 +1531,8 @@ settings_page_proxy (struct session *sess, GtkWidget * vbox)
 {
 	GtkWidget *wid;
 	GtkWidget *tab;
+	GtkWidget *pauth_group;
+	GtkWidget *pauth_tab;
 	gint row_index;
 	static char *proxymenutext[] =
 	{
@@ -1528,7 +1554,23 @@ settings_page_proxy (struct session *sess, GtkWidget * vbox)
 		settings_create_numberentry (_("Proxy Server Port:"), 0, 65535, 1, tab,
 								prefs.proxy_port, row_index++, 0);
 	settings_create_optmenu (tab, _("Proxy Type:"), proxymenutext,
-								settings_proxytypeset, sess, row_index, prefs.proxy_type);
+								settings_proxytypeset, sess, row_index++, prefs.proxy_type);
+	pauth_group = settings_create_group (wid, _("Proxy Authorization (HTTP proxy only)"));
+	settings_create_toggle (_("Proxy requires authorization"), pauth_group,
+					    prefs.proxy_auth,
+					    settings_proxy_auth_check,
+					    sess);
+	pauth_tab = settings_create_table (pauth_group);
+	row_index = 0;
+	settings_create_entry (_("Proxy User:"),
+								  sizeof (prefs.proxy_user) - 1, pauth_tab, prefs.proxy_user,
+								  &sess->setup->entry_proxy_user, 0, 0, row_index++);
+	settings_create_entry (_("Proxy Password:"),
+								  sizeof (prefs.proxy_pass) - 1, pauth_tab, prefs.proxy_pass,
+								  &sess->setup->entry_proxy_pass, 0, 0, row_index);
+	gtk_entry_set_visibility(GTK_ENTRY(sess->setup->entry_proxy_pass), FALSE);
+	gtk_widget_set_sensitive (sess->setup->entry_proxy_user, prefs.proxy_auth);
+	gtk_widget_set_sensitive (sess->setup->entry_proxy_pass, prefs.proxy_auth);
 }
 
 static void
