@@ -65,6 +65,7 @@
 #define GTK_OBJECT_CLASS(n) ((GtkObjectClass*)n)
 
 #ifdef WIN32
+#include <windows.h>
 #define charlen(str) g_utf8_skip[*(guchar *)(str)]
 #else
 #define charlen(str) mblen(str, MB_CUR_MAX)
@@ -99,7 +100,7 @@ int get_stamp_str (time_t, char *, int);
 #endif
 static void gtk_xtext_render_page (GtkXText * xtext);
 static void gtk_xtext_calc_lines (GtkXText * xtext, int);
-#ifdef USE_XLIB
+#if defined(USE_XLIB) || defined(WIN32)
 static void gtk_xtext_load_trans (GtkXText * xtext);
 static void gtk_xtext_free_trans (GtkXText * xtext);
 #endif
@@ -213,9 +214,11 @@ gtk_xtext_init (GtkXText * xtext)
 
 		gtk_selection_add_targets (GTK_WIDGET (xtext), GDK_SELECTION_PRIMARY,
 											targets, n_targets);
+#ifdef WIN32
 		gtk_selection_add_targets (GTK_WIDGET (xtext),
 											gdk_atom_intern ("CLIPBOARD", FALSE),
 											targets, n_targets);
+#endif
 	}
 	gtk_signal_connect (GTK_OBJECT (xtext), "selection_get",
 							  GTK_SIGNAL_FUNC (gtk_xtext_selection_get), xtext);
@@ -326,7 +329,7 @@ gtk_xtext_destroy (GtkObject * object)
 
 	if (xtext->pixmap)
 	{
-#ifdef USE_XLIB
+#if defined(USE_XLIB) || defined(WIN32)
 		if (xtext->transparent)
 			gtk_xtext_free_trans (xtext);
 		else
@@ -458,7 +461,7 @@ gtk_xtext_realize (GtkWidget * widget)
 	xtext_set_bg (xtext->fgc, xtext->palette[19]);
 	xtext_set_fg (xtext->bgc, xtext->palette[19]);
 
-#ifdef USE_XLIB
+#if defined(USE_XLIB) || defined(WIN32)
 	if (xtext->transparent)
 	{
 		gtk_xtext_load_trans (xtext);
@@ -499,18 +502,24 @@ static void
 gtk_xtext_size_allocate (GtkWidget * widget, GtkAllocation * allocation)
 {
 	GtkXText *xtext = GTK_XTEXT (widget);
+	int height_only = FALSE;
 
-	if (allocation->width == widget->allocation.width &&
-		 allocation->height == widget->allocation.height)
-		return;
+	if (allocation->width == widget->allocation.width)
+		height_only = TRUE;
 
 	widget->allocation = *allocation;
 	if (GTK_WIDGET_REALIZED (widget))
 	{
 		gdk_window_move_resize (widget->window, allocation->x, allocation->y,
 										allocation->width, allocation->height);
-		gtk_xtext_calc_lines (xtext, FALSE);
-#ifdef USE_XLIB
+		if (!height_only)
+			gtk_xtext_calc_lines (xtext, FALSE);
+		else
+		{
+			xtext->pagetop_ent = NULL;
+			gtk_xtext_adjustment_set (xtext, FALSE);
+		}
+#if defined(USE_XLIB) || defined(WIN32)
 		if (xtext->transparent && xtext->shaded)
 		{
 			gtk_xtext_free_trans (xtext);
@@ -531,7 +540,7 @@ static void
 gtk_xtext_draw (GtkWidget * widget, GdkRectangle * area)
 {
 	GtkXText *xtext = GTK_XTEXT (widget);
-#ifdef USE_XLIB
+#if defined(USE_XLIB) || defined(WIN32)
 	int x, y;
 
 	if (xtext->transparent)
@@ -714,10 +723,6 @@ gtk_xtext_find_char (GtkXText * xtext, int x, int y, int *off,
 	textentry *ent;
 	int line;
 	int subline;
-	int win_width;
-
-	gdk_window_get_size (GTK_WIDGET (xtext)->window, &win_width, 0);
-	win_width -= MARGIN;
 
 	line = (y - xtext->font->descent + xtext->pixel_offset) / xtext->fontsize;
 
@@ -732,6 +737,50 @@ gtk_xtext_find_char (GtkXText * xtext, int x, int y, int *off,
 	return ent;
 }
 
+#ifdef WIN32
+
+static void
+win32_draw_bg (GtkXText *xtext, int x, int y, int width, int height)
+{
+	HDC hdc;
+	HWND hwnd;
+	HRGN rgn;
+
+	if (xtext->shaded)
+	{
+		gdk_draw_drawable (xtext->draw_buf, xtext->bgc, xtext->pixmap,
+								 x, y, x, y, width, height);
+	} else
+	{
+		hwnd = GDK_DRAWABLE_XID (xtext->draw_buf);
+		hdc = GetDC (hwnd);
+
+		rgn = CreateRectRgn (x, y, x + width, y + height);
+		SelectClipRgn (hdc, rgn);
+
+		PaintDesktop (hdc);
+
+		ReleaseDC (hwnd, hdc);
+		DeleteObject (rgn);
+	}
+}
+
+static void
+xtext_draw_bg (GtkXText *xtext, int x, int y, int width, int height)
+{
+	if (xtext->transparent)
+		win32_draw_bg (xtext, x, y, width, height);
+	else
+		gdk_draw_rectangle (xtext->draw_buf, xtext->bgc, 1, x, y, width, height);
+}
+
+#else
+
+#define xtext_draw_bg(xt,x,y,w,h) gdk_draw_rectangle(xt->draw_buf, xt->bgc, \
+																	  1,x,y,w,h);
+
+#endif
+
 static gint
 gtk_xtext_expose (GtkWidget * widget, GdkEventExpose * event)
 {
@@ -752,9 +801,8 @@ gtk_xtext_expose (GtkWidget * widget, GdkEventExpose * event)
 		return FALSE;
 	}
 
-	gdk_draw_rectangle (xtext->draw_buf, xtext->bgc, 1,
-							  event->area.x, event->area.y,
-							  event->area.width, event->area.height);
+	xtext_draw_bg (xtext, event->area.x, event->area.y,
+						event->area.width, event->area.height);
 
 	ent_start = gtk_xtext_find_char (xtext, event->area.x, event->area.y,
 												NULL, NULL);
@@ -1085,9 +1133,11 @@ gtk_xtext_leave_notify (GtkWidget * widget, GdkEventCrossing * event)
 		gdk_window_set_cursor (widget->window, 0);
 		xtext->skip_border_fills = TRUE;
 		xtext->do_underline_fills_only = TRUE;
+		xtext->skip_stamp = TRUE;
 		gtk_xtext_render_ents (xtext, xtext->hilight_ent, NULL, FALSE);
 		xtext->skip_border_fills = FALSE;
 		xtext->do_underline_fills_only = FALSE;
+		xtext->skip_stamp = FALSE;
 		xtext->hilight_ent = NULL;
 	}
 	return FALSE;
@@ -1164,8 +1214,10 @@ gtk_xtext_motion_notify (GtkWidget * widget, GdkEventMotion * event)
 				xtext->hilight_ent = word_ent;
 				xtext->hilight_start = offset;
 				xtext->hilight_end = offset + len;
+
 				xtext->skip_border_fills = TRUE;
 				xtext->do_underline_fills_only = TRUE;
+				xtext->skip_stamp = TRUE;
 
 				if (old_ent != word_ent)
 					xtext->jump_out_early = TRUE;
@@ -1175,6 +1227,7 @@ gtk_xtext_motion_notify (GtkWidget * widget, GdkEventMotion * event)
 				xtext->jump_out_early = FALSE;
 				xtext->skip_border_fills = FALSE;
 				xtext->do_underline_fills_only = FALSE;
+				xtext->skip_stamp = FALSE;
 			}
 			return FALSE;
 		}
@@ -1190,8 +1243,10 @@ gtk_xtext_motion_notify (GtkWidget * widget, GdkEventMotion * event)
 static void
 gtk_xtext_set_clip_owner (GtkWidget * xtext, GdkEventButton * event)
 {
+#ifdef WIN32
 	gtk_selection_owner_set (xtext, gdk_atom_intern ("CLIPBOARD", FALSE),
 									 event->time);
+#endif
 	gtk_selection_owner_set (xtext, GDK_SELECTION_PRIMARY, event->time);
 }
 
@@ -1237,9 +1292,11 @@ gtk_xtext_button_release (GtkWidget * widget, GdkEventButton * event)
 			if (gtk_xtext_selection_clear (xtext))
 			{
 				xtext->skip_border_fills = TRUE;
+				xtext->skip_stamp = TRUE;
 				gtk_xtext_render_ents (xtext, xtext->last_ent_start,
 											  xtext->last_ent_end, TRUE);
 				xtext->skip_border_fills = FALSE;
+				xtext->skip_stamp = FALSE;
 
 				xtext->last_ent_start = NULL;
 				xtext->last_ent_end = NULL;
@@ -1283,6 +1340,7 @@ gtk_xtext_button_press (GtkWidget * widget, GdkEventButton * event)
 		return FALSE;
 	}
 
+#ifndef WIN32
 	if (event->button == 4)		  /* mouse wheel pageUp */
 	{
 		new_value = xtext->adj->value - xtext->adj->page_increment;
@@ -1300,6 +1358,7 @@ gtk_xtext_button_press (GtkWidget * widget, GdkEventButton * event)
 		gtk_adjustment_set_value (xtext->adj, new_value);
 		return FALSE;
 	}
+#endif
 
 	if (event->button == 2)
 	{
@@ -1322,12 +1381,14 @@ gtk_xtext_button_press (GtkWidget * widget, GdkEventButton * event)
 			ent->mark_end = offset + len;
 			/* clear the old mark, if any */
 			xtext->skip_border_fills = TRUE;
+			xtext->skip_stamp = TRUE;
 			gtk_xtext_render_ents (xtext, xtext->last_ent_start, xtext->last_ent_end, TRUE);
 			/* render the word select */
 			xtext->last_ent_start = ent;
 			xtext->last_ent_end = ent;
 			gtk_xtext_render_ents (xtext, ent, NULL, FALSE);
 			xtext->skip_border_fills = FALSE;
+			xtext->skip_stamp = FALSE;
 			xtext->word_or_line_select = TRUE;
 			gtk_xtext_set_clip_owner (GTK_WIDGET (xtext), event);
 		}
@@ -1345,12 +1406,14 @@ gtk_xtext_button_press (GtkWidget * widget, GdkEventButton * event)
 			ent->mark_end = ent->str_len;
 			/* clear the old mark, if any */
 			xtext->skip_border_fills = TRUE;
+			xtext->skip_stamp = TRUE;
 			gtk_xtext_render_ents (xtext, xtext->last_ent_start, xtext->last_ent_end, TRUE);
 			/* render the line select */
 			xtext->last_ent_start = ent;
 			xtext->last_ent_end = ent;
 			gtk_xtext_render_ents (xtext, ent, NULL, FALSE);
 			xtext->skip_border_fills = FALSE;
+			xtext->skip_stamp = FALSE;
 			xtext->word_or_line_select = TRUE;
 			gtk_xtext_set_clip_owner (GTK_WIDGET (xtext), event);
 		}
@@ -1365,7 +1428,7 @@ gtk_xtext_button_press (GtkWidget * widget, GdkEventButton * event)
 		if (line_x == x || line_x == x + 1 || line_x == x - 1)
 		{
 			xtext->moving_separator = TRUE;
-			gtk_xtext_render_page (xtext);
+			gtk_xtext_draw_sep (xtext, -1);
 			return FALSE;
 		}
 	}
@@ -1388,9 +1451,16 @@ gtk_xtext_selection_kill (GtkWidget * widget, GdkEventSelection * event)
 #ifndef WIN32
 	if (gtk_xtext_selection_clear (GTK_XTEXT (widget)))
 	{
+		GTK_XTEXT (widget)->skip_border_fills = TRUE;
+		GTK_XTEXT (widget)->skip_stamp = TRUE;
+		gtk_xtext_render_ents (GTK_XTEXT (widget),
+									  GTK_XTEXT (widget)->last_ent_start,
+									  GTK_XTEXT (widget)->last_ent_end, TRUE);
+		GTK_XTEXT (widget)->skip_border_fills = FALSE;
+		GTK_XTEXT (widget)->skip_stamp = FALSE;
+
 		GTK_XTEXT (widget)->last_ent_start = NULL;
 		GTK_XTEXT (widget)->last_ent_end = NULL;
-		gtk_xtext_render_page (GTK_XTEXT (widget));
 	}
 #endif
 	return TRUE;
@@ -1493,6 +1563,44 @@ gtk_xtext_selection_get (GtkWidget * widget,
 	free (stripped);
 }
 
+#ifdef WIN32
+
+static gint
+gtk_xtext_scroll (GtkWidget *widget, GdkEventScroll *event)
+{
+	GtkXText *xtext = GTK_XTEXT (widget);
+
+	if (event->direction == GDK_SCROLL_UP)
+	{
+		if (xtext->adj->value > 0.0)
+		{
+			xtext->adj->value -= 3;
+			if (xtext->adj->value < 0.0)
+				xtext->adj->value = 0.0;
+			gtk_adjustment_changed (xtext->adj);
+			gtk_xtext_render_page (xtext);
+		}
+		return TRUE;
+	}
+
+	if (event->direction == GDK_SCROLL_DOWN)
+	{
+		if (xtext->adj->value < xtext->adj->upper - xtext->adj->page_size)
+		{
+			xtext->adj->value += 3;
+			if (xtext->adj->value > xtext->adj->upper - xtext->adj->page_size)
+				xtext->adj->value = xtext->adj->upper - xtext->adj->page_size;
+			gtk_adjustment_changed (xtext->adj);
+			gtk_xtext_render_page (xtext);
+		}
+		return TRUE;
+	}
+
+	return FALSE;
+}
+
+#endif
+
 static void
 gtk_xtext_class_init (GtkXTextClass * class)
 {
@@ -1529,6 +1637,9 @@ gtk_xtext_class_init (GtkXTextClass * class)
 #endif
 	widget_class->draw = gtk_xtext_draw;
 	widget_class->expose_event = gtk_xtext_expose;
+#ifdef WIN32
+	widget_class->scroll_event = gtk_xtext_scroll;
+#endif
 
 	xtext_class->word_click = NULL;
 }
@@ -1810,15 +1921,13 @@ gtk_xtext_render_flush (GtkXText * xtext, int x, int y, unsigned char *str,
 			/* redraw the background at the descent (e.g. redraws the
 				background where the j's and y's end). This is used for
 				unrendering a highlight. */
-			gdk_draw_rectangle (GTK_WIDGET (xtext)->window, xtext->bgc, 1,
-									  x, y + 1, str_width, 1);
+			xtext_draw_bg (xtext, x, y + 1, str_width, 1);
 			dofill = FALSE;
 		} else if (xtext->pixmap)
 		{
 	/* draw the background pixmap behind the text - CAUSES FLICKER HERE!! */
-			gdk_draw_rectangle (GTK_WIDGET (xtext)->window, xtext->bgc, 1,
-									  x, y - xtext->font->ascent, str_width,
-							 		  xtext->fontsize);
+			xtext_draw_bg (xtext, x, y - xtext->font->ascent, str_width,
+								xtext->fontsize);
 			dofill = FALSE;	/* non pixmap bg is done by XDrawImageString */
 		}
 	}
@@ -1871,13 +1980,21 @@ gtk_xtext_render_flush (GtkXText * xtext, int x, int y, unsigned char *str,
 	/* don't have Xlib, gdk version --- */
 	if (dofill)
 	{
-		GdkGCValues val;
-		gdk_gc_get_values (gc, &val);
-		xtext_set_fg (gc, val.background.pixel);
-		gdk_draw_rectangle (xtext->draw_buf, gc, 1,
-								x, y - xtext->font->ascent, str_width,
-							 	xtext->fontsize);
-		xtext_set_fg (gc, val.foreground.pixel);
+#ifdef WIN32
+		if (xtext->transparent && !xtext->backcolor)
+			win32_draw_bg (xtext, x, y - xtext->font->ascent, str_width,
+								xtext->fontsize);
+		else
+#endif
+		{
+			GdkGCValues val;
+			gdk_gc_get_values (gc, &val);
+			xtext_set_fg (gc, val.background.pixel);
+			gdk_draw_rectangle (xtext->draw_buf, gc, 1,
+									x, y - xtext->font->ascent, str_width,
+								 	xtext->fontsize);
+			xtext_set_fg (gc, val.foreground.pixel);
+		}
 	}
 	gdk_draw_text (xtext->draw_buf, xtext->font, gc, x, y, str, len);
 	if (xtext->bold)
@@ -1964,15 +2081,14 @@ gtk_xtext_render_str (GtkXText * xtext, int y, textentry * ent, unsigned char *s
 			/* don't overwrite the timestamp */
 			if (indent > xtext->stamp_width)
 			{
-				gdk_draw_rectangle (xtext->draw_buf, xtext->bgc, 1, 
-								 xtext->stamp_width, y - xtext->font->ascent,
-								 indent - xtext->stamp_width, xtext->fontsize);
+				xtext_draw_bg (xtext, xtext->stamp_width, y - xtext->font->ascent,
+									indent - xtext->stamp_width, xtext->fontsize);
 			}
 		} else
 		{
 			/* fill the indent area with background gc */
-			gdk_draw_rectangle (xtext->draw_buf, xtext->bgc, 1, 0,
-									y - xtext->font->ascent, indent, xtext->fontsize);
+			xtext_draw_bg (xtext, 0, y - xtext->font->ascent, indent,
+								xtext->fontsize);
 		}
 	}
 
@@ -2169,9 +2285,8 @@ gtk_xtext_render_str (GtkXText * xtext, int y, textentry * ent, unsigned char *s
 	gtk_xtext_draw_sep (xtext, y - xtext->font->ascent);
 	/* draw background to the right of the text */
 	if (!xtext->skip_border_fills)
-		gdk_draw_rectangle (xtext->draw_buf, xtext->bgc, 1, 
-						 x, y - xtext->font->ascent, (win_width + MARGIN) - x,
-						 xtext->fontsize);
+		xtext_draw_bg (xtext, x, y - xtext->font->ascent,
+							(win_width + MARGIN) - x, xtext->fontsize);
 
 	return ret;
 }
@@ -2457,6 +2572,7 @@ shade_pixmap (GtkXText * xtext, Pixmap p, int x, int y, int w, int h)
 }
 
 #endif /* !USE_TINT */
+#endif /* !USE_XLIB */
 
 /* free transparency xtext->pixmap */
 
@@ -2465,6 +2581,9 @@ gtk_xtext_free_trans (GtkXText * xtext)
 {
 	if (xtext->pixmap)
 	{
+#ifdef WIN32
+		gdk_pixmap_unref (xtext->pixmap);
+#else
 		if (xtext->shaded)
 		{
 			gdk_pixmap_unref (xtext->pixmap);
@@ -2474,15 +2593,78 @@ gtk_xtext_free_trans (GtkXText * xtext)
 			g_dataset_destroy (xtext->pixmap);
 			g_free (xtext->pixmap);
 		}
+#endif
 		xtext->pixmap = NULL;
 	}
 }
 
+#ifdef WIN32
+
+static GdkPixmap *
+win32_tint (GtkXText *xtext, GdkImage *img, int width, int height)
+{
+	int x, y;
+	GdkPixmap *pix;
+	GdkVisual *visual = img->visual;
+	gulong pixel;
+	int r, g, b;
+
+	for (y = 0; y < height; y++)
+	{
+		for (x = 0; x < width; x++)
+		{
+			pixel = gdk_image_get_pixel (img, x, y);
+
+			r = (pixel & visual->red_mask) >> visual->red_shift;
+			g = (pixel & visual->green_mask) >> visual->green_shift;
+			b = (pixel & visual->blue_mask) >> visual->blue_shift;
+
+			gdk_image_put_pixel (img, x, y,
+						((r * xtext->tint_red) >> 8) << visual->red_shift |
+						((g * xtext->tint_green) >> 8) << visual->green_shift |
+						((b * xtext->tint_blue) >> 8) << visual->blue_shift);
+		}
+	}
+
+	if (xtext->recycle)
+		pix = xtext->pixmap;
+	else
+		pix = gdk_pixmap_new (GTK_WIDGET (xtext)->window, width, height,
+									 xtext->depth);
+	gdk_draw_image (pix, xtext->bgc, img, 0, 0, 0, 0, width, height);
+
+	return pix;
+}
+
+#endif /* !WIN32 */
+
 /* grab pixmap from root window and set xtext->pixmap */
 
 static void
 gtk_xtext_load_trans (GtkXText * xtext)
 {
+#ifdef WIN32
+	GdkImage *img;
+	int width, height;
+	HDC hdc;
+	HWND hwnd;
+
+	/* if not shaded, we paint directly with PaintDesktop() */
+	if (!xtext->shaded)
+		return;
+
+	hwnd = GDK_DRAWABLE_XID (GTK_WIDGET (xtext)->window);
+	hdc = GetDC (hwnd);
+	PaintDesktop (hdc);
+	ReleaseDC (hwnd, hdc);
+
+	gdk_window_get_size (GTK_WIDGET (xtext)->window, &width, &height);
+	img = gdk_image_get (GTK_WIDGET (xtext)->window, 0, 0, width, height);
+	xtext->pixmap = win32_tint (xtext, img, width, height);
+	gdk_image_destroy (img);
+
+#else
+
 	Pixmap rootpix;
 	GtkWidget *widget = GTK_WIDGET (xtext);
 	int x, y;
@@ -2524,10 +2706,9 @@ noshade:
 	}
 #endif
 	gdk_gc_set_fill (xtext->bgc, GDK_TILED);
+#endif	/* !WIN32 */
 }
 
-#endif
-
 /* walk through str until this line doesn't fit anymore */
 
 static int
@@ -2848,18 +3029,18 @@ gtk_xtext_set_background (GtkXText * xtext, GdkPixmap * pixmap, int trans,
 {
 	GdkGCValues val;
 
-#if !defined(USE_GDK_PIXBUF) && !defined(USE_MMX)
+#if !defined(USE_GDK_PIXBUF) && !defined(USE_MMX) && !defined(WIN32)
 	shaded = FALSE;
 #endif
 
-#ifndef USE_XLIB
+#if !defined(USE_XLIB) && !defined(WIN32)
 	shaded = FALSE;
 	trans = FALSE;
 #endif
 
 	if (xtext->pixmap)
 	{
-#ifdef USE_XLIB
+#if defined(USE_XLIB) || defined(WIN32)
 		if (xtext->transparent)
 			gtk_xtext_free_trans (xtext);
 		else
@@ -2870,7 +3051,7 @@ gtk_xtext_set_background (GtkXText * xtext, GdkPixmap * pixmap, int trans,
 
 	xtext->transparent = trans;
 
-#ifdef USE_XLIB
+#if defined(USE_XLIB) || defined(WIN32)
 	if (trans)
 	{
 		xtext->shaded = shaded;
@@ -3195,8 +3376,7 @@ gtk_xtext_render_page (GtkXText * xtext)
 
 	line = (xtext->fontsize * line) - xtext->pixel_offset;
 	/* fill any space below the last line with our background GC */
-	gdk_draw_rectangle (xtext->draw_buf, xtext->bgc, 1,
-						 0, line, width + MARGIN, height - line);
+	xtext_draw_bg (xtext, 0, line, width + MARGIN, height - line);
 
 	/* draw the separator line */
 	gtk_xtext_draw_sep (xtext, -1);
@@ -3207,7 +3387,7 @@ gtk_xtext_refresh (GtkXText * xtext, int do_trans)
 {
 	if (GTK_WIDGET_REALIZED (GTK_WIDGET (xtext)))
 	{
-#ifdef USE_XLIB
+#if defined(USE_XLIB) || defined(WIN32)
 		if (xtext->transparent && do_trans)
 		{
 			gtk_xtext_free_trans (xtext);
